import logging
from collections import defaultdict
from typing import Any, Dict, List, Tuple, Union

from repro.common import TemporaryDirectory, util
from repro.common.docker import make_volume_map, run_command
from repro.common.io import write_to_text_file
from repro.data.types import MetricsType, TextType
from repro.models import Model

from .commands import sentence_split

logger = logging.getLogger(__name__)


@Model.register("lin2004-rouge")
class ROUGE(Model):
    def __init__(
        self,
        image: str = "lin2004",
        ngram_order: int = 4,
        porter_stemmer: bool = True,
        remove_stopwords: bool = False,
        sentence_split: bool = True,
        calculate_su4: bool = True,
    ):
        self.image = image
        self.ngram_order = ngram_order
        self.porter_stemmer = porter_stemmer
        self.remove_stopwords = remove_stopwords
        self.sentence_split = sentence_split
        self.calculate_su4 = calculate_su4

    def _maybe_sentence_split(self, texts: List[TextType]) -> List[List[str]]:
        if any(isinstance(text, str) for text in texts):
            if not all(isinstance(text, str) for text in texts):
                raise Exception(
                    f"Input texts are mixed types between strings and lists of strings. "
                    f"All must be of the same type"
                )
            return sentence_split(self.image, texts)
        else:
            return texts

    def _maybe_sentence_split_texts_list(
        self, texts_list: List[List[TextType]]
    ) -> List[List[List[str]]]:
        # Flatten the texts into a single list so we can call `_maybe_sentence_split`, then
        # rearrange the output to be parallel to `texts_list`
        flat_texts = []
        for texts in texts_list:
            flat_texts.extend(texts)

        split_texts = self._maybe_sentence_split(flat_texts)
        split_texts_list = []
        index = 0
        for texts in texts_list:
            split_texts_list.append([])
            for _ in texts:
                split_texts_list[-1].append(split_texts[index])
                index += 1
        return split_texts_list

    @staticmethod
    def _write_to_text_file(text: Union[str, List[str]], file_path: str) -> None:
        # We wrap the one text in a list, then write the whole list to a file. If
        # `text` is a `str`, then it will only write 1 line. If it's a `List[str]`,
        # it will be one sentence per line
        write_to_text_file([text], file_path, separator="\n")

    @staticmethod
    def _write_config_file(
        candidate_filenames_list: List[List[str]],
        reference_filenames_list: List[List[str]],
        file_path: str,
    ) -> None:
        with open(file_path, "w") as out:
            out.write(f'<ROUGE_EVAL version="1.0">\n')
            for i, (candidate_filenames, reference_filenames) in enumerate(
                zip(candidate_filenames_list, reference_filenames_list)
            ):
                out.write(f'<EVAL ID="{i + 1}">\n')
                out.write(f'<INPUT-FORMAT TYPE="SPL"></INPUT-FORMAT>\n')
                out.write(f"<PEER-ROOT>/</PEER-ROOT>\n")
                out.write(f"<MODEL-ROOT>/</MODEL-ROOT>\n")

                out.write(f"<PEERS>\n")
                for j, candidate_filename in enumerate(candidate_filenames):
                    out.write(f'<P ID="{j + 1}">{candidate_filename}</P>\n')
                out.write(f"</PEERS>\n")

                out.write(f"<MODELS>\n")
                for j, reference_filename in enumerate(reference_filenames):
                    symbol = chr(j + 65)
                    out.write(f'<M ID="{symbol}">{reference_filename}</M>\n')
                out.write(f"</MODELS>\n")

                out.write(f"</EVAL>\n")
            out.write(f"</ROUGE_EVAL>\n")

    @staticmethod
    def _parse_individual_line(
        columns: List[str],
    ) -> Tuple[int, int, float, float, float]:
        assert len(columns) == 7
        period = columns[3].index(".")
        group_index = int(columns[3][:period]) - 1
        candidate_index = int(columns[3][period + 1 :]) - 1
        recall = float(columns[4][2:]) * 100
        precision = float(columns[5][2:]) * 100
        f1 = float(columns[6][2:]) * 100
        return group_index, candidate_index, recall, precision, f1

    @staticmethod
    def _parse_stdout(stdout: str) -> Dict[int, Dict[int, Dict]]:
        lines = stdout.splitlines()
        # metrics_dict[group_index][candidate_index][metric] = value
        metrics_dict = defaultdict(lambda: defaultdict(dict))
        for line in lines:
            if line in [
                "---------------------------------------------",
                ".............................................",
            ]:
                continue
            columns = line.split()
            rouge_metric = columns[1].lower()
            if columns[2] == "Eval":
                (
                    group_index,
                    candidate_index,
                    recall,
                    precision,
                    f1,
                ) = ROUGE._parse_individual_line(columns)
                metrics_dict[group_index][candidate_index][rouge_metric] = {
                    "recall": recall,
                    "precision": precision,
                    "f1": f1,
                }
        return metrics_dict

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating ROUGE for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # ROUGE can be quite slow, so we deduplicate processing by grouping all of
        # the sources by identical targets. There could be duplicate references if you are scoring multiple
        # systems outputs at once. We then process the data grouped, and ungroup it at the end
        # so the output scores are parallel to the inputs.
        (
            grouped_candidates_list,
            grouped_references_list,
            group_mapping,
        ) = util.group_by_references(candidates, references_list)

        if self.sentence_split:
            grouped_candidates_list = self._maybe_sentence_split_texts_list(
                grouped_candidates_list
            )
            grouped_references_list = self._maybe_sentence_split_texts_list(
                grouped_references_list
            )

        # The ROUGE config file requires pointing to the candidate and reference filenames
        candidate_filenames_list = []
        reference_filenames_list = []

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]

            # Serialize all of the texts to disk
            for i, (candidates, references) in enumerate(
                zip(grouped_candidates_list, grouped_references_list)
            ):
                candidate_filenames = []
                for j, candidate in enumerate(candidates):
                    candidate_filename = f"{i}/model.{j}.txt"
                    candidate_filenames.append(
                        f"{container_input_dir}/{candidate_filename}"
                    )
                    self._write_to_text_file(
                        candidate, f"{host_input_dir}/{candidate_filename}"
                    )
                candidate_filenames_list.append(candidate_filenames)

                reference_filenames = []
                for j, reference in enumerate(references):
                    symbol = chr(j + 65)
                    reference_filename = f"{i}/gold.{symbol}.txt"
                    reference_filenames.append(
                        f"{container_input_dir}/{reference_filename}"
                    )
                    self._write_to_text_file(
                        reference, f"{host_input_dir}/{reference_filename}"
                    )
                reference_filenames_list.append(reference_filenames)

            # Save the config file
            host_config_file = f"{host_input_dir}/config.xml"
            container_config_file = f"{container_input_dir}/config.xml"
            self._write_config_file(
                candidate_filenames_list, reference_filenames_list, host_config_file
            )

            # Run the command
            command = (
                f"perl ROUGE-1.5.5/ROUGE-1.5.5.pl"
                f"  -e ROUGE-1.5.5/data"
                f"  -n {self.ngram_order}"
                f"  -a"
                f"  -c 95"
                f"  -r 1000"
                f"  -p 0.5"
                f"  -t 0"
                f"  -d"
            )
            if self.porter_stemmer:
                command += " -m"
            if self.remove_stopwords:
                command += " -s"
            if self.calculate_su4:
                command += " -2 4 -u"
            command += f" {container_config_file}"

            stdout = run_command(
                self.image,
                command,
                volume_map=volume_map,
                network_disabled=True,
                stderr=False,
                silent=True,
            )

            metrics_dict = self._parse_stdout(stdout)
            micro_metrics = util.ungroup_values(metrics_dict, group_mapping)
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics



import json
import unittest

from parameterized import parameterized

from repro.models.lewis2020 import BART
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT


class TestLewis2020Models(unittest.TestCase):
    def setUp(self) -> None:
        self.expected_output = json.load(open(f"{FIXTURES_ROOT}/expected-output.json"))

    @parameterized.expand(get_testing_device_parameters())
    def test_bart_cnn(self, device: int):
        model = BART(device=device)
        document = self.expected_output["CNNDM"]["document"]
        expected_summary = self.expected_output["CNNDM"]["summary"]
        summary = model.predict(document)
        assert summary == expected_summary

    @parameterized.expand(get_testing_device_parameters())
    def test_bart_xsum(self, device: int):
        model = BART("bart.large.xsum", device=device)
        document = self.expected_output["XSum"]["document"]
        expected_summary = self.expected_output["XSum"]["summary"]
        summary = model.predict(document)
        assert summary == expected_summary



import json
import logging
from typing import Dict, List, Optional, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("zhao2019-moverscore")
class MoverScore(Model):
    def __init__(self, image: str = "zhao2019", device: int = 0):
        self.image = image
        self.device = device

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self,
        inputs: List[Dict[str, Union[TextType, List[TextType]]]],
        batch_size: Optional[int] = None,
        use_stopwords: bool = False,
        **kwargs,
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(
            f"Calculating MoverScore with image {self.image} on {len(inputs)} inputs."
        )

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # Each candidate and reference must be `str`, not `List[str]`
        candidates = [util.flatten(candidate) for candidate in candidates]
        references_list = [
            [util.flatten(reference) for reference in references]
            for references in references_list
        ]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, references in zip(candidates, references_list):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "references": references,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            cuda = self.device != -1
            commands = []
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")

            score_command = (
                f"python score.py"
                f"  --input-file {container_input_file}"
                f"  --use-stopwords {str(use_stopwords).lower()}"
                f"  --output-file {container_output_file}"
            )
            if batch_size is not None:
                score_command += f" --batch-size {batch_size}"
            commands.append(score_command)

            command = " && ".join(commands)
            backend.run_command(command=command, cuda=cuda)

            micro = read_jsonl_file(host_output_file)
            macro = util.average_dicts(micro)
            return macro, micro


@Model.register("zhao2019-moverscore-summarization")
class MoverScoreForSummarization(MoverScore):
    def predict_batch(self, *args, **kwargs) -> Tuple[MetricsType, List[MetricsType]]:
        if "use_stopwords" in kwargs:
            if kwargs["use_stopwords"] != True:
                raise Exception(
                    f"`use_stopwords` must be `True` for `MoverScoreForSummarization`"
                )
        else:
            kwargs["use_stopwords"] = True
        return super().predict_batch(*args, **kwargs)



import logging
from typing import Dict, List, Union

from repro.data.types import DocumentType, SummaryType
from repro.models import Model, SingleDocumentSummarizationModel
from repro.models.liu2019 import BertSumExt

from .commands import get_oracle_sentences, generate_summaries, sentence_split

logger = logging.getLogger(__name__)


@Model.register("dou2021-oracle-sentence-gsum")
class OracleSentenceGSumModel(Model):
    def __init__(
        self, image: str = "dou2021", device: int = 0, batch_size: int = 16
    ) -> None:
        self.model = "bart_sentence"
        self.image = image
        self.device = device
        self.batch_size = batch_size

    def predict(
        self,
        document: DocumentType,
        reference: SummaryType = None,
        guidance: SummaryType = None,
        **kwargs,
    ) -> SummaryType:
        if reference is None and guidance is None:
            raise Exception(f"Either `reference` or `guidance` must be provided")
        return self.predict_batch(
            [{"document": document, "reference": reference, "guidance": guidance}]
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[DocumentType, SummaryType]]], **kwargs
    ) -> List[SummaryType]:
        logger.info(
            f"Generating summaries for {len(inputs)} inputs and image {self.image}."
        )

        # Check if all inputs have "guidance" provided. If they do, we will use that
        # instead of computing the guidance
        def _has_guidance(inp: Dict[str, Union[DocumentType, SummaryType]]) -> bool:
            return "guidance" in inp and inp["guidance"] is not None

        compute_guidance = True
        if any(_has_guidance(inp) for inp in inputs):
            if not all(_has_guidance(inp) for inp in inputs):
                raise Exception(
                    f"Only some of the inputs have `guidance` inputs. Only all or none is supported"
                )
            logger.info("Using input guidance")
            compute_guidance = False
        else:
            logger.info("Computing guidance from references")

        # If the documents and references are pre-sentence split, we will maintain
        # that split. Otherwise, we run sentence splitting
        documents = [inp["document"] for inp in inputs]
        if any(isinstance(document, str) for document in documents):
            if any(isinstance(document, list) for document in documents):
                logger.warning(
                    "`documents` contains both sentence-split and un-sentence-split documents. "
                    "The sentence-split boundaries will be ignored and sentence splitting will "
                    "be run again."
                )
            tokenized_documents = sentence_split(self.image, documents)
        else:
            tokenized_documents = documents

        if compute_guidance:
            references = [inp["reference"] for inp in inputs]
            if any(isinstance(reference, str) for reference in references):
                # Sentence splitting needs to be run
                if any(isinstance(reference, list) for reference in references):
                    logger.warning(
                        "`references` contains both sentence-split and un-sentence-split references. "
                        "The sentence-split boundaries will be ignored and sentence splitting will "
                        "be run again."
                    )
                references = sentence_split(self.image, references)
            guidance = get_oracle_sentences(self.image, tokenized_documents, references)
        else:
            guidance = [inp["guidance"] for inp in inputs]

        # We pass the original, untokenized documents to the generation.
        summaries = generate_summaries(
            self.image, self.model, self.device, self.batch_size, documents, guidance
        )
        return summaries


@Model.register("dou2021-sentence-gsum")
class SentenceGSumModel(SingleDocumentSummarizationModel):
    def __init__(
        self, image: str = "dou2021", device: int = 0, batch_size: int = 16
    ) -> None:
        self.model = "bart_sentence"
        self.image = image
        self.device = device
        self.batch_size = batch_size
        self.extractive_model = BertSumExt(device=device)

    def predict_batch(
        self, inputs: List[Dict[str, Union[DocumentType, SummaryType]]], *args, **kwargs
    ) -> List[SummaryType]:
        logger.info(
            f"Generating summaries for {len(inputs)} inputs and image {self.image}."
        )
        documents = [inp["document"] for inp in inputs]

        # The sentence supervision is done using the `self.extractive_model`, which
        # is in its own Docker container, so we retrieve those summaries first
        logger.info(f"Extracting guidance signal")
        guidance = self.extractive_model.predict_batch(inputs)

        summaries = generate_summaries(
            self.image, self.model, self.device, self.batch_size, documents, guidance
        )
        return summaries



import argparse
import inspect
import json
from overrides import overrides
from typing import Any, Dict, List, Union

from repro.commands.subcommand import RootSubcommand
from repro.commands.util import load_dataset_reader, load_model, load_output_writer
from repro.common.logging import prepare_global_logging
from repro.data.dataset_readers import HuggingfaceDatasetsDatasetReader
from repro.data.types import InstanceDict
from repro.models import Model


def predict_with_model(
    model: Model,
    instances: List[InstanceDict],
    kwargs: Union[str, Dict[str, Any]] = None,
) -> Any:
    # Deserialize kwargs if necessary
    kwargs = kwargs or {}
    if isinstance(kwargs, str):
        kwargs = json.loads(kwargs)
    if not isinstance(kwargs, dict):
        raise ValueError(
            f"`kwargs` is expected to be a dictionary or json-serialized dictionary: {kwargs}"
        )

    # Find the required arguments for the model's `predict` function
    parameters = inspect.signature(model.predict).parameters
    required_args = set()
    for key, value in parameters.items():
        if value.kind == inspect.Parameter.POSITIONAL_ONLY:
            required_args.add(key)

    # Ensure all of the instances have at least those arguments
    for i, instance in enumerate(instances):
        for arg in required_args:
            if arg not in instance:
                raise Exception(f"Instance {i} missing required argument {arg}")

    # Pass all of the instances, including with other, non-required arguments
    predictions = model.predict_batch(instances, **kwargs)
    return predictions


@RootSubcommand.register("predict")
class PredictSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = "Predict using a model"
        self.parser = parser.add_parser(
            "predict", description=description, help=description
        )
        self.parser.add_argument(
            "--model-name", required=True, help="The name of the model to predict with"
        )
        self.parser.add_argument(
            "--model-kwargs",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the model constructor",
        )
        self.parser.add_argument(
            "--predict-kwargs",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the `Model.predict_batch` function",
        )
        self.parser.add_argument(
            "--dataset-name",
            required=False,
            help="The name of the Huggingface `datasets` library dataset to predict on",
        )
        self.parser.add_argument(
            "--split",
            required=False,
            help="The split of the Huggingface `datasets` dataset to use",
        )
        self.parser.add_argument(
            "--input-files",
            required=False,
            nargs="+",
            help="The input file(s) to pass to the dataset reader",
        )
        self.parser.add_argument(
            "--dataset-reader",
            required=False,
            help="The name of the dataset reader to use",
        )
        self.parser.add_argument(
            "--dataset-reader-kwargs",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the dataset reader constructor",
        )
        self.parser.add_argument(
            "--output",
            required=True,
            help="The name of the output file or directory that the model predictions will be written to",
        )
        self.parser.add_argument(
            "--output-writer",
            required=False,
            default="default",
            help="The name of the class to use to write the predictions to the output file",
        )
        self.parser.add_argument(
            "--output-writer-kwargs",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the output writer constructor",
        )
        self.parser.add_argument(
            "--log-file",
            required=False,
            help="The file where the log should be written",
        )
        self.parser.add_argument(
            "--silent",
            required=False,
            action="store_true",
            help="Indicates the log should not be written to stdout",
        )
        self.parser.set_defaults(func=self.run)

    @staticmethod
    def _check_args(args):
        # --dataset-name and --input-files are exclusive-or
        if (args.dataset_name is not None) is (args.input_files is not None):
            raise ValueError(
                "Exactly one of --dataset-name or --input-files must be set"
            )

        # Both --dataset-name and --split must either be set or not set
        if (args.dataset_name is not None) is not (args.split is not None):
            raise ValueError(
                "Parameters --dataset-name and --split must either both be set "
                "or neither be set"
            )

        # Both --input-files and --dataset-reader must either be set or not set
        if (args.input_files is not None) is not (args.dataset_reader is not None):
            raise ValueError(
                "Parameters --input-files and --dataset-reader must either both be set "
                "or neither be set"
            )

        # If --dataset-reader-kwargs is passed, --dataset-reader must also be used
        if args.dataset_reader_kwargs is not None:
            if args.dataset_reader is None:
                raise ValueError(
                    "Parameter --dataset-reader must be used if --dataset-reader-kwargs "
                    "is also used"
                )

    @overrides
    def run(self, args):
        self._check_args(args)
        prepare_global_logging(args.log_file, args.silent)

        model = load_model(args.model_name, args.model_kwargs)

        if args.dataset_name is not None:
            dataset_reader = HuggingfaceDatasetsDatasetReader(
                args.dataset_name, args.split
            )
            instances = dataset_reader.read()
        else:
            dataset_reader = load_dataset_reader(
                args.dataset_reader, args.dataset_reader_kwargs
            )
            instances = dataset_reader.read(*args.input_files)

        predictions = predict_with_model(model, instances, args.predict_kwargs)

        output_writer = load_output_writer(
            args.output_writer, args.output_writer_kwargs
        )
        output_writer.write(
            instances, predictions, args.output, model_name=args.model_name
        )



from typing import Any, Dict, List, Union

DocumentType = Union[str, List[str]]
InstanceDict = Dict[str, Any]
MetricsType = Dict[str, Union[float, "MetricsType"]]
SummaryType = Union[str, List[str]]
TextType = Union[str, List[str]]



import argparse

from overrides import overrides

from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand, build_image


@SetupSubcommand.register("zhang2020")
class Zhang2020SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("zhang2020", f"{MODELS_ROOT}/zhang2020")

    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = f'Build the docker image "{self.image}"'
        self.parser = parser.add_parser(
            self.image, description=description, help=description
        )
        self.parser.add_argument(
            "--models",
            nargs="+",
            help="Indicates which models should be downloaded",
            default=["roberta-large"],
        )
        self.parser.add_argument(
            "--silent",
            action="store_true",
            help="Silences the output from the build command",
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        build_args = {
            "MODELS": " ".join(args.models),
        }
        build_image(self.root, self.image, build_args=build_args, silent=args.silent)



import argparse

from repro.common import Registrable


class Subcommand(Registrable):
    def add_subparser(self, parser: argparse._SubParsersAction) -> None:
        raise NotImplementedError

    def run(self, args):
        raise NotImplementedError


# These empty subcommand classes are used to be able to automatically add the
# subcommands to the argument parser in the right locations. For example, any command
# which is registered under the `RootSubcommand` will become a command directly under "repro"
class RootSubcommand(Subcommand):
    pass


class SetupSubcommand(Subcommand):
    pass



import json
import unittest

from parameterized import parameterized

from repro.models.liu2019 import BertSumExt, BertSumExtAbs, TransformerAbs
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT


class TestLiu2019Models(unittest.TestCase):
    def setUp(self) -> None:
        # The example documents were taken from the PreSumm repo:
        # https://github.com/nlpyang/PreSumm/blob/dev/raw_data/temp.raw_src
        self.expected_output = json.load(open(f"{FIXTURES_ROOT}/expected-output.json"))

    @parameterized.expand(get_testing_device_parameters())
    def test_bert_sum_ext_regression(self, device: int):
        model = BertSumExt(device=device)

        # Test when the document is pre-sentence split
        document = self.expected_output["BertSumExt"]["document"]
        expected_summary = self.expected_output["BertSumExt"]["summary"]
        summary = model.predict(document)
        assert summary == expected_summary

    @parameterized.expand(get_testing_device_parameters())
    def test_bert_sum_ext_abs_cnndm_regression(self, device: int):
        model = BertSumExtAbs(device=device)
        document = self.expected_output["BertSumExtAbs_CNNDM"]["document"]
        expected_summary = self.expected_output["BertSumExtAbs_CNNDM"]["summary"]
        summary = model.predict(document)
        assert summary == expected_summary

    @parameterized.expand(get_testing_device_parameters())
    def test_bert_sum_ext_abs_xsum_regression(self, device: int):
        model = BertSumExtAbs(model="bertsumextabs_xsum.pt", device=device)
        document = self.expected_output["BertSumExtAbs_XSum"]["document"]
        expected_summary = self.expected_output["BertSumExtAbs_XSum"]["summary"]
        summary = model.predict(document)
        assert summary == expected_summary

    @parameterized.expand(get_testing_device_parameters())
    def test_transformer_abs_regression(self, device: int):
        model = TransformerAbs(device=device)
        document = self.expected_output["TransformerAbs"]["document"]
        expected_summary = self.expected_output["TransformerAbs"]["summary"]
        summary = model.predict(document)
        assert summary == expected_summary



from typing import Any, List

from repro.common import Registrable
from repro.data.types import InstanceDict


class OutputWriter(Registrable):
    def __init__(self, prediction_per_instance: bool) -> None:
        """
        Parameters
        ----------
        prediction_per_instance : bool
            If `True`, indicates there should be exactly one prediction per instances, and
            the `write()` method will verify this is the case. If `False`, no checking is done.
        """
        self.prediction_per_instance = prediction_per_instance

    def write(
        self,
        instances: List[InstanceDict],
        predictions: Any,
        output_file_or_dir: str,
        *args,
        **kwargs,
    ) -> None:
        if self.prediction_per_instance:
            if not isinstance(predictions, list):
                raise Exception(
                    f"`predictions` is expected to be a list if `self.prediction_per_instance` is `True`"
                )
            if len(instances) != len(predictions):
                raise Exception(
                    f"Number of instances {len(instances)} is not equal to the number "
                    f"of predictions {len(predictions)}"
                )
        self._write(instances, predictions, output_file_or_dir, *args, **kwargs)

    def _write(
        self,
        instances: List[InstanceDict],
        predictions: Any,
        output_file_or_dir: str,
        *args,
        **kwargs,
    ) -> None:
        """
        Writes the results of the prediction to the `output_file_or_dir`. The `instances` and `predictions`
        are parallel lists such that `predictions[i]` is the prediction for `instances[i]`.

        Parameters
        ----------
        instances : List[InstanceDict]
            The instances that were used to get the predictions
        predictions : Any
            The predictions to save
        output_file_or_dir : str
            The output file or directory
        """
        raise NotImplementedError



import json
import unittest
from parameterized import parameterized

from repro.models.scialom2021 import (
    QuestEval,
    QuestEvalForSummarization,
    QuestEvalForSimplification,
)
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


class TestScialom2021Models(unittest.TestCase):
    def setUp(self) -> None:
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))
        self.questeval_examples = json.load(
            open(f"{FIXTURES_ROOT}/questeval-unittests.json", "r")
        )

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_summarization_reference_and_source(self, device: int):
        model = QuestEvalForSummarization(device=device)
        # We only take the first reference and source since QuestEval only supports one of each
        inputs = [
            {
                "candidate": inp["candidate"],
                "references": [inp["references"][0]],
                "sources": [inp["sources"][0]],
            }
            for inp in self.examples
        ]
        expected_macro = self.expected["reference_and_source"]["macro"]
        expected_micro = self.expected["reference_and_source"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_summarization_reference_only(self, device: int):
        model = QuestEvalForSummarization(device=device)
        # We only take the first reference and source since QuestEval only supports one of each
        inputs = [
            {"candidate": inp["candidate"], "references": [inp["references"][0]]}
            for inp in self.examples
        ]
        expected_macro = self.expected["reference_only"]["macro"]
        expected_micro = self.expected["reference_only"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_summarization_source_only(self, device: int):
        model = QuestEvalForSummarization(device=device)
        # We only take the first reference and source since QuestEval only supports one of each
        inputs = [
            {"candidate": inp["candidate"], "sources": [inp["sources"][0]]}
            for inp in self.examples
        ]
        expected_macro = self.expected["source_only"]["macro"]
        expected_micro = self.expected["source_only"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    def test_questeval_summarization_invalid_kwargs(self):
        model = QuestEvalForSummarization()
        with self.assertRaises(Exception):
            model.predict(candidate="Candidate", sources=["Source"], task="text2text")

        with self.assertRaises(Exception):
            model.predict(candidate="Candidate", sources=["Source"], do_weighter=False)

        with self.assertRaises(Exception):
            model.predict_batch([], task="text2text")

        with self.assertRaises(Exception):
            model.predict_batch([], do_weighter=False)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_unittests_reference_and_source(self, device: int):
        # Tests an example from the QuestEval repo with the reference and source.
        # The candidate and source and swapped in the repo
        model = QuestEval(device=device)
        inp = self.questeval_examples["summarization"]["reference_and_source"]["input"]
        expected = self.questeval_examples["summarization"]["reference_and_source"][
            "output"
        ]
        actual = model.predict(**inp)
        assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_unittests_reference_only(self, device: int):
        # Tests an example from the QuestEval repo with the reference only.
        # The candidate and source and swapped in the repo
        model = QuestEval(device=device)
        inp = self.questeval_examples["summarization"]["reference_only"]["input"]
        expected = self.questeval_examples["summarization"]["reference_only"]["output"]
        actual = model.predict(**inp)
        assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_unittests_source_only(self, device: int):
        # Tests an example from the QuestEval repo with the source only.
        # The candidate and source and swapped in the repo
        model = QuestEval(device=device)
        inp = self.questeval_examples["summarization"]["source_only"]["input"]
        expected = self.questeval_examples["summarization"]["source_only"]["output"]
        actual = model.predict(**inp)
        assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_simplification_reference_and_source_regression(
        self, device: int
    ):
        # Tests an example from the QuestEval repo but applied to simplification
        model = QuestEvalForSimplification(device=device)
        inp = self.questeval_examples["simplification"]["reference_and_source"]["input"]
        expected = self.questeval_examples["simplification"]["reference_and_source"][
            "output"
        ]
        actual = model.predict(**inp)
        assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_simplification_reference_only_regression(self, device: int):
        # Tests an example from the QuestEval repo but applied to simplification
        model = QuestEvalForSimplification(device=device)
        inp = self.questeval_examples["simplification"]["reference_only"]["input"]
        expected = self.questeval_examples["simplification"]["reference_only"]["output"]
        actual = model.predict(**inp)
        assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_questeval_simplification_source_only_regression(self, device: int):
        # Tests an example from the QuestEval repo but applied to simplification
        model = QuestEvalForSimplification(device=device)
        inp = self.questeval_examples["simplification"]["source_only"]["input"]
        expected = self.questeval_examples["simplification"]["source_only"]["output"]
        actual = model.predict(**inp)
        assert_dicts_approx_equal(expected, actual, abs=1e-4)

    def test_questeval_simplification_invalid_kwargs(self):
        model = QuestEvalForSimplification()
        with self.assertRaises(Exception):
            model.predict(candidate="Candidate", sources=["Source"], task="text2text")

        with self.assertRaises(Exception):
            model.predict(candidate="Candidate", sources=["Source"], do_BERTScore=False)

        with self.assertRaises(Exception):
            model.predict_batch([], task="text2text")

        with self.assertRaises(Exception):
            model.predict_batch([], do_BERTScore=False)



import argparse
import json
import os

from qaeval import QuestionGenerationModel


def main(args):
    inputs = []
    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            inputs.append((data["context"], data["start"], data["end"]))

    model = QuestionGenerationModel(args.model_file, args.cuda_device, args.batch_size)
    questions = model.generate_all(inputs)

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, "w") as out:
        for question in questions:
            out.write(json.dumps({"question": question}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--model-file", required=True)
    argp.add_argument("--cuda-device", type=int, required=True)
    argp.add_argument("--batch-size", type=int, required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand


@SetupSubcommand.register("squad-v2")
class SQuADv2SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("squad-v2", f"{MODELS_ROOT}/squad_v2")



import argparse
import json
import numpy as np

from moverscore_v2 import get_idf_dict, word_mover_score


def main(args):
    candidates = []
    references_list = []

    unique_candidates = set()
    unique_references = set()

    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            candidate = data["candidate"]
            references = data["references"]
            candidates.append(candidate)
            references_list.append(references)
            unique_candidates.add(candidate)
            for reference in references:
                unique_references.add(reference)

    # Calculate the idf dicts on the unique texts
    idf_dict_candidates = get_idf_dict(list(unique_candidates))
    idf_dict_references = get_idf_dict(list(unique_references))

    # Optionally load the stopwords
    if args.use_stopwords.lower() == "true":
        with open("stopwords.txt", "r", encoding="utf-8") as f:
            stopwords = set(f.read().strip().split(" "))
    else:
        stopwords = []

    # Flatten the candidates and references to be passed through `word_mover_score` all
    # at once for faster processing
    flat_candidates = []
    flat_references = []
    for candidate, references in zip(candidates, references_list):
        for reference in references:
            flat_candidates.append(candidate)
            flat_references.append(reference)

    scores = word_mover_score(
        flat_references,
        flat_candidates,
        idf_dict_references,
        idf_dict_candidates,
        stop_words=stopwords,
        n_gram=1,
        remove_subwords=True,
        batch_size=args.batch_size,
    )

    with open(args.output_file, "w") as out:
        # Take the mean over the references for the candidate
        index = 0
        for references in references_list:
            this_scores = []
            for _ in references:
                this_scores.append(scores[index])
                index += 1
            average_score = np.mean(this_scores)
            out.write(json.dumps({"moverscore": average_score}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--use-stopwords", required=True)
    argp.add_argument("--batch-size", type=int, default=48)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand


@SetupSubcommand.register("zhao2019")
class Zhao2019SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("zhao2019", f"{MODELS_ROOT}/zhao2019")



import logging
import os
from glob import glob
from typing import Dict, List

from overrides import overrides

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import write_to_text_file
from repro.models import Model, SingleDocumentSummarizationModel
from repro.models.model import DocumentType, SummaryType

logger = logging.getLogger(__name__)


class _Liu2019Model(SingleDocumentSummarizationModel):
    """
    A wrapper around the TransformerAbs, BertSumExt, and BertSumExtAbs models proposed
    in `Liu & Lapata (2019) <https://arxiv.org/abs/1908.08345>`.
    """

    def __init__(
        self,
        model: str,
        task: str,
        image: str = "liu2019",
        device: int = 0,
        min_length: int = None,
        max_length: int = None,
        alpha: float = None,
    ) -> None:
        """
        Parameters
        ----------
        model : str
            The name of the pretrained model
        task : str
            The task, either "ext" for an extractive model or "abs" for an abstractive model
        image : str, default="liu2019"
            The name of the Docker image
        device : int, default=0
            The ID of the GPU to use, -1 if CPU
        min_length : int, default=None
            The minimum length of the summary. If `None`, defaults to the original code
        max_length : int, default=None
            The maximum length of the summary. If `None`, defaults to the original code
        alpha : float, default=None
            The alpha length penalty parameter. If `None`, defaults to the original code
        """
        self.model = model
        self.task = task
        self.image = image
        self.device = device
        self.min_length = min_length
        self.max_length = max_length
        self.alpha = alpha

    @staticmethod
    def _load_results(output_dir: str) -> List[SummaryType]:
        # The summaries are written to the output_dir in a file with the
        # extension ".candidate". Here, we find that file and load it
        file_paths = glob(f"{output_dir}/*.candidate")
        if len(file_paths) != 1:
            raise Exception(f"Found {len(file_paths)} output files. Expected 1.")
        return open(file_paths[0], "r").read().splitlines()

    @staticmethod
    def _sentence_split_summaries(summaries: List[str]) -> List[List[str]]:
        return [summary.split("<q>") for summary in summaries]

    @overrides
    def predict_batch(
        self, inputs: List[Dict[str, DocumentType]], *args, **kwargs
    ) -> List[SummaryType]:
        documents = [inp["document"] for inp in inputs]
        logger.info(
            f"Predicting summaries for {len(documents)} documents "
            f"with pretrained model {self.model}, task {self.task} "
            f"and Docker image {self.image}."
        )

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/documents.txt"
            container_input_file = f"{container_input_dir}/documents.txt"
            write_to_text_file(documents, host_input_file)

            # Run inference. The output_dir must exist before running the docker command
            os.makedirs(host_output_dir)
            container_tokenized_file = f"{container_output_dir}/tokenized.txt"
            container_output_prefix = f"{container_output_dir}/out"

            commands = []
            commands.append(
                f"python preprocess.py"
                f"  --input-file {container_input_file}"
                f"  --output-file {container_tokenized_file}"
            )
            commands.append("cd PreSumm/src")

            # The train.py script sets the CUDA_VISIBLE_DEVICES environment variable,
            # so we do not do it here
            train_command = (
                f"python train.py"
                f"  -task {self.task}"
                f"  -mode test_text"
                f"  -test_from ../../{self.model}"
                f"  -text_src {container_tokenized_file}"
                f"  -result_path {container_output_prefix}"
                f"  -visible_gpus {self.device}"
            )
            if self.max_length is not None:
                train_command += f" -max_length {self.max_length}"
            if self.min_length is not None:
                train_command += f" -min_length {self.min_length}"
            if self.alpha is not None:
                train_command += f" -alpha {self.alpha}"
            commands.append(train_command)

            command = " && ".join(commands)

            cuda = self.device != -1
            run_command(
                self.image,
                command,
                volume_map=volume_map,
                network_disabled=True,
                cuda=cuda,
            )

            # Load the output summaries
            summaries = self._load_results(host_output_dir)

            # The <q> token splits sentences in each of the summaries.
            # Split them into sentences
            summaries = self._sentence_split_summaries(summaries)
            return summaries


@Model.register("liu2019-bertsumext")
class BertSumExt(_Liu2019Model):
    def __init__(self, **kwargs) -> None:
        super().__init__(model="bertsumext_cnndm.pt", task="ext", **kwargs)


@Model.register("liu2019-bertsumextabs")
class BertSumExtAbs(_Liu2019Model):
    def __init__(self, model: str = "bertsumextabs_cnndm.pt", **kwargs) -> None:
        # Default values are taken from the Readme in the original repo
        if model == "bertsumextabs_cnndm.pt":
            min_length = kwargs.pop("min_length", 50)
            max_length = kwargs.pop("max_length", 200)
            alpha = kwargs.pop("alpha", 0.95)
        elif model == "bertsumextabs_xsum.pt":
            min_length = kwargs.pop("min_length", 20)
            max_length = kwargs.pop("max_length", 100)
            alpha = kwargs.pop("alpha", 0.9)
        else:
            raise Exception(f"Unknown pretrained model: {model}")

        super().__init__(
            model=model,
            task="abs",
            min_length=min_length,
            max_length=max_length,
            alpha=alpha,
            **kwargs,
        )


@Model.register("liu2019-transformerabs")
class TransformerAbs(_Liu2019Model):
    def __init__(self, **kwargs) -> None:
        # Default values are taken from the Readme in the original repo
        min_length = kwargs.pop("min_length", 50)
        max_length = kwargs.pop("max_length", 200)
        alpha = kwargs.pop("alpha", 0.95)

        super().__init__(
            model="transformerabs_cnndm.pt",
            task="abs",
            min_length=min_length,
            max_length=max_length,
            alpha=alpha,
            **kwargs,
        )



import json
import os
from overrides import overrides
from typing import Any, List

from repro.data.output_writers import OutputWriter
from repro.data.types import InstanceDict


@OutputWriter.register("default")
class DefaultOutputWriter(OutputWriter):
    """
    Writes a jsonl file with keys for the `instance_id`, `model_id`, and `prediction`.
    """

    def __init__(self, include_input: bool = False):
        super().__init__(True)
        self.include_input = include_input

    @overrides
    def _write(
        self,
        instances: List[InstanceDict],
        predictions: Any,
        output_file_or_dir: str,
        model_name: str,
        *args,
        **kwargs
    ) -> None:
        output_file = output_file_or_dir
        dirname = os.path.dirname(output_file)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        with open(output_file, "w") as out:
            for instance, prediction in zip(instances, predictions):
                data = {
                    "instance_id": instance["instance_id"],
                    "model_id": model_name,
                }
                if self.include_input:
                    data["input"] = instance
                data["prediction"] = prediction

                out.write(json.dumps(data) + "\n")



import json
import pytest
import unittest

from repro.models.papineni2002 import BLEU, SentBLEU
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal

from . import FIXTURES_ROOT


class TestPapineni2002Models(unittest.TestCase):
    def setUp(self) -> None:
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))

    def test_sent_bleu(self):
        model = SentBLEU()
        inputs = [
            {"candidate": inp["candidate"], "references": inp["references"]}
            for inp in self.examples
        ]
        expected_macro = self.expected["sentbleu"]["macro"]
        expected_micro = self.expected["sentbleu"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    def test_bleu(self):
        model = BLEU()
        inputs = [
            {"candidate": inp["candidate"], "references": inp["references"]}
            for inp in self.examples
        ]
        expected_macro = self.expected["bleu"]["macro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(actual_micro) == 0

    def test_bleu_sacrebleu_tests(self):
        # Tests an example from the sacrebleu unit tests
        model = BLEU()
        inputs = [
            {
                "candidate": "The dog bit the man.",
                "references": ["The dog bit the man.", "The dog had bit the man."],
            },
            {
                "candidate": "It wasn't surprising.",
                "references": ["It was not unexpected.", "No one was surprised."],
            },
            {
                "candidate": "The man had just bitten him.",
                "references": ["The man bit him first.", "The man had bitten the dog."],
            },
        ]
        expected = 48.530827
        actual_macro, actual_micro = model.predict_batch(inputs)
        assert actual_macro["bleu"] == pytest.approx(expected, abs=1e-4)
        assert len(actual_micro) == 0

    def test_bleu_unequal_references_regression(self):
        # Tests an example when the candidates have an unequal number of references
        model = BLEU()
        inputs = [
            {
                "candidate": "The dog bit the man.",
                "references": ["The dog bit the man.", "The dog had bit the man."],
            },
            {
                "candidate": "It wasn't surprising.",
                "references": ["No one was surprised."],
            },
            {
                "candidate": "The man had just bitten him.",
                "references": [
                    "The man bit him first.",
                    "The man had bitten the dog.",
                    "The man bit the dog before.",
                ],
            },
        ]
        expected = 47.63997460581784
        actual_macro, actual_micro = model.predict_batch(inputs)
        assert actual_macro["bleu"] == pytest.approx(expected, abs=1e-4)
        assert len(actual_micro) == 0



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand


@SetupSubcommand.register("deutsch2021")
class Deutsch2021SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("deutsch2021", f"{MODELS_ROOT}/deutsch2021")



import json
from typing import Any, Dict, T, Type, Union

from repro.common import Registrable
from repro.data.dataset_readers import DatasetReader
from repro.data.output_writers import OutputWriter
from repro.models import Model


def _load_type(
    base_type: Type[T], name: str, kwargs: Union[str, Dict[str, Any]] = None
) -> T:
    kwargs = kwargs or {}
    if isinstance(kwargs, str):
        kwargs = json.loads(kwargs)
    if not isinstance(kwargs, dict):
        raise ValueError(
            f"`kwargs` is expected to be a dictionary or json-serialized dictionary: {kwargs}"
        )

    type_, _ = Registrable._registry[base_type][name]
    obj = type_(**kwargs)
    return obj


def load_model(name: str, kwargs: Union[str, Dict[str, Any]] = None) -> Model:
    """
    Loads a `Model` given the registered `name` of the model and any arguments
    which will be passed to the constructor as kwargs. `args` should be a dictionary
    or a json-serialized dictionary in which the keys correspond to constructor parameters.

    Parameters
    ----------
    name : str
        The name of the model to load
    kwargs : Union[str, Dict[str, Any]]
        The kwargs to be passed to the model's constructor

    Returns
    -------
    Model
        The model
    """
    return _load_type(Model, name, kwargs)


def load_dataset_reader(
    name: str, kwargs: Union[str, Dict[str, Any]] = None
) -> DatasetReader:
    """
    Loads a `DatasetReader` given the registered `name` of the reader and any arguments
    which will be passed to the constructor as kwargs. `args` should be a dictionary
    or a json-serialized dictionary in which the keys correspond to constructor parameters.

    Parameters
    ----------
    name : str
        The name of the dataset reader to load
    kwargs : Union[str, Dict[str, Any]]
        The kwargs to be passed to the dataset reader's constructor

    Returns
    -------
    DatasetReader
        The dataset reader
    """
    return _load_type(DatasetReader, name, kwargs)


def load_output_writer(
    name: str, kwargs: Union[str, Dict[str, Any]] = None
) -> DatasetReader:
    """
    Loads an `OutputWriter` given the registered `name` of the reader and any arguments
    which will be passed to the constructor as kwargs. `args` should be a dictionary
    or a json-serialized dictionary in which the keys correspond to constructor parameters.

    Parameters
    ----------
    name : str
        The name of the output writer to load
    kwargs : Union[str, Dict[str, Any]]
        The kwargs to be passed to the output writer's constructor

    Returns
    -------
    OutputWriter
        The output writer
    """
    return _load_type(OutputWriter, name, kwargs)



from overrides import overrides
from typing import List

from repro.common.io import read_jsonl_file
from repro.data.dataset_readers import DatasetReader
from repro.data.types import InstanceDict


@DatasetReader.register("deutsch2021-question-answering-eval")
class Deutsch2021QuestionAnsweringEvaluationDatasetReader(DatasetReader):
    @overrides
    def _read(self, *input_files: str) -> List[InstanceDict]:
        instances = []
        for input_file in input_files:
            predictions = read_jsonl_file(input_file)
            for prediction in predictions:
                instances.append(
                    {
                        "instance_id": prediction["instance_id"],
                        "prediction": prediction["prediction"]["prediction"],
                        "null_probability": prediction["prediction"][
                            "null_probability"
                        ],
                    }
                )
        return instances



import argparse
import docker
import json
import logging
import shutil
import tempfile
from typing import Dict, Optional

from overrides import overrides

from repro.commands.subcommand import SetupSubcommand
from repro.common import REPRO_CONFIG
from repro.common.logging import prepare_global_logging

logger = logging.getLogger(__name__)


def make_volume_map(*volumes: str) -> Dict[str, str]:
    volumes_dict = {}
    for i, volume in enumerate(volumes):
        volumes_dict[volume] = f"/tmp{i}"
    return volumes_dict


def image_exists(image: str) -> bool:
    try:
        client = docker.from_env()
    except docker.errors.DockerException as e:
        logger.error("Could not connect to the Docker client. Is the Daemon running?")
        logger.exception(e)
        return False
    try:
        client.images.get(image)
    except docker.errors.ImageNotFound:
        return False
    return True


def remove_image(image: str, force: bool = False) -> None:
    client = docker.from_env()
    client.images.remove(image, force)


def run_command(
    image: str,
    command: str,
    volume_map: Optional[Dict[str, str]] = None,
    stdout: bool = True,
    stderr: bool = True,
    silent: bool = False,
    network_disabled: bool = False,
    cuda: bool = False,
) -> str:
    """
    Runs a shell command in a Docker image.

    Parameters
    ----------
    image : str
        The name of the Docker image
    command : str
        The command to run
    volume_map : Dict[str, str], default=None
        A mapping between host directories to container directories
    stdout : bool, default=True
        Indicates the stdout stream should be returned
    stderr : bool, default=True
        Indicates the stderr stream should be returned
    silent : bool, default=False
        Indicates whether the stdout and stderr streams of the command should be written to stdout
        if `stdout` and `stderr` are `True`, respectively
    network_disabled : bool, default=False
        Indicates the container's network connection should be disabled
    cuda : bool, default=False
        Indicates that the processes uses cuda, in which the runtime will be set to "nvidia"

    Returns
    -------
    str
        The stdout and stderr streams of the command if `stdout` and `stderr` are `True`, respectively.
        There appears to be a bug in the `docker` package related to collecting the full stdout and stderr streams, so
        we do not recommend relying on these streams.
    """
    if not stdout and not stderr:
        raise ValueError(
            f"The `docker` package requires either `stdout` or `stderr` is `True`"
        )

    volume_map = volume_map or {}
    volumes = {
        host_path: {"bind": container_path, "mode": "rw"}
        for host_path, container_path in volume_map.items()
    }

    runtime = "nvidia" if cuda else None

    # Escape any single quotes from the command
    command = command.replace("'", "'\\''")

    docker_command = f"/bin/sh -c '{command}'"
    logger.info(f'Running command in Docker image {image}: "{docker_command}"')

    client = docker.from_env()
    container = client.containers.run(
        image,
        docker_command,
        volumes=volumes,
        detach=True,
        network_disabled=network_disabled,
        runtime=runtime,
    )
    logs = container.logs(stream=True, stdout=stdout, stderr=stderr)

    # Collect the stdout and/or stderr, printing if verbose
    output = []
    for item in logs:
        item = item.decode()
        output.append(item)
        if not silent:
            print(item, end="")
    logger.info("Command finished")

    output = "".join(output)
    return output


def build_image(
    root: str, image: str, build_args: Dict[str, str] = None, silent: bool = False
) -> None:
    logger.info(f"Building image {image} with Dockerfile in directory {root}")
    docker_server = REPRO_CONFIG["docker_server"]
    client = docker.APIClient(base_url=docker_server)
    generator = client.build(path=root, tag=image, buildargs=build_args, rm=True)
    try:
        for response in generator:
            response = response.decode()
            for json_obj in response.split("\r\n"):
                if json_obj:
                    data = json.loads(json_obj)
                    if "stream" in data:
                        stream = data["stream"]
                        if not silent:
                            print(stream, end="")
    except Exception as e:
        logger.error("Error building image")
        logger.exception(e)

    logger.info("Finished building image")


class BuildDockerImageSubcommand(SetupSubcommand):
    def __init__(
        self,
        image: str,
        root: str,
    ) -> None:
        self.image = image
        self.root = root

    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = f'Build the docker image "{self.image}"'
        self.parser = parser.add_parser(
            self.image, description=description, help=description
        )
        self.parser.add_argument(
            "--silent",
            action="store_true",
            help="Silences the output from the build command",
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        prepare_global_logging(silent=args.silent)
        build_image(self.root, self.image, silent=args.silent)


class DockerContainer(object):
    def __init__(self, image: str):
        self.image = image

    def __enter__(self):
        self.host_dir = tempfile.mkdtemp()
        self.volume_map = make_volume_map(self.host_dir)
        self.container_dir = self.volume_map[self.host_dir]
        return self

    def __exit__(self, *args):
        shutil.rmtree(self.host_dir)

    def run_command(self, **kwargs) -> str:
        for arg in ["image", "volume_map"]:
            if arg in kwargs:
                raise Exception(
                    f"`{arg}` parameter cannot be passed to the `DockerContainer`"
                    f"`run_command` function."
                )
        return run_command(self.image, volume_map=self.volume_map, **kwargs)



import logging
import os
from typing import List

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file, write_to_jsonl_file

logger = logging.getLogger(__name__)


def sentence_split(image: str, inputs: List[str]) -> List[List[str]]:
    """
    Run sentence splitting on the inputs.

    Parameters
    ----------
    image : str
        The name of the SacreROUGE Docker image to use.
    inputs : List[str]
        The texts which should be sentence-split.
    Returns
    -------
    List[List[str]]
        The sentence-split text.
    """
    with TemporaryDirectory() as temp:
        host_input_dir = f"{temp}/input"
        host_output_dir = f"{temp}/output"
        volume_map = make_volume_map(host_input_dir, host_output_dir)
        container_input_dir = volume_map[host_input_dir]
        container_output_dir = volume_map[host_output_dir]

        host_input_file = f"{host_input_dir}/input.txt"
        host_output_file = f"{host_output_dir}/output.txt"
        container_input_file = f"{container_input_dir}/input.txt"
        container_output_file = f"{container_output_dir}/output.txt"
        write_to_jsonl_file(inputs, "text", host_input_file)

        command = (
            f"python sentence_split.py {container_input_file} {container_output_file}"
        )
        os.makedirs(host_output_dir)
        run_command(image, command, volume_map=volume_map)

        outputs = read_jsonl_file(host_output_file)
        sentences = [output["sentences"] for output in outputs]
        return sentences



import argparse
import json
import os

import bert_score


def main(args):
    candidates = []
    references_list = []

    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            candidates.append(data["candidate"])
            references_list.append(data["references"])

    device = None if args.cuda_device == -1 else args.cuda_device
    precisions, recall, f1s = bert_score.score(
        candidates,
        references_list,
        model_type=args.model_name,
        device=device,
        batch_size=args.batch_size,
        lang=args.language,
    )

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, "w") as out:
        for precision, recall, f1 in zip(precisions, recall, f1s):
            out.write(
                json.dumps(
                    {
                        "precision": precision.item(),
                        "recall": recall.item(),
                        "f1": f1.item(),
                    }
                )
                + "\n"
            )


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--model-name")
    argp.add_argument("--cuda-device", required=True, type=int)
    argp.add_argument("--batch-size", type=int, default=64)
    argp.add_argument("--language")
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



from repro.data.dataset_readers.dataset_reader import DatasetReader
from repro.data.dataset_readers.datasets import HuggingfaceDatasetsDatasetReader
from repro.data.dataset_readers.jsonl import JSONLinesDatasetReader
from repro.data.dataset_readers.sacrerouge import SacreROUGEDatasetReader



import argparse
import json
import os
from sacrebleu import corpus_bleu


def main(args):
    # Load the data
    candidates = []
    references_list = []
    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            candidates.append(data["candidate"])
            references_list.append(data["references"])

    # sacrebleu requires an equal number of references for
    # each candidate, but you can use `None` for padding
    max_references = max(len(references) for references in references_list)
    for references in references_list:
        references.extend([None] * (max_references - len(references)))

    # The references are expected to be inverted. That is, references_list[i][j]
    # is the i-th reference for the j-th candidate
    inverted_references_list = [[] for _ in range(max_references)]
    for references in references_list:
        for i, reference in enumerate(references):
            inverted_references_list[i].append(reference)

    # Calculate BLEU
    result = corpus_bleu(candidates, inverted_references_list)

    # Save the result
    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)
    with open(args.output_file, "w") as out:
        out.write(json.dumps({"bleu": result.score}))


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



"""
Adapted from https://github.com/kirubarajan/roft/blob/master/generation/interactive_test.py to
process a batch of inputs.
"""
import argparse
import json
import numpy as np
import os
import torch
from transformers import AutoModelForCausalLM, AutoTokenizer


def main(args):
    np.random.seed(args.random_seed)
    torch.manual_seed(args.random_seed)
    if torch.cuda.is_available():
        torch.cuda.manual_seed_all(args.random_seed)

    tokenizer = AutoTokenizer.from_pretrained(args.model_name)
    model = AutoModelForCausalLM.from_pretrained(args.model_name)

    if torch.cuda.is_available():
        model = model.cuda()

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in f:
                data = json.loads(line)
                name = data["name"]
                ingredients = "\n".join(data["ingredients"])

                input_text = f"HOW TO MAKE: {name}\nIngredients:\n{ingredients}."
                input_tensor = tokenizer.encode(input_text, return_tensors="pt").to(
                    model.device
                )
                outputs = model.generate(
                    input_tensor,
                    do_sample=True,
                    top_p=args.top_p,
                    repetition_penalty=args.repetition_penalty,
                    pad_token_id=tokenizer.eos_token_id,
                    max_length=args.max_length,
                )
                recipe = [tokenizer.decode(x) for x in outputs][0]
                out.write(json.dumps({"recipe": recipe}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--model-name", required=True)
    argp.add_argument("--top-p", type=float, default=0.7)
    argp.add_argument("--repetition-penalty", type=float, default=1.2)
    argp.add_argument("--max-length", type=int, default=256)
    argp.add_argument("--random-seed", type=int, default=4)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import json
import os
from overrides import overrides
from typing import Any, List

from repro.data.output_writers import OutputWriter
from repro.data.types import InstanceDict


@OutputWriter.register("metrics")
class MetricsOutputWriter(OutputWriter):
    """
    Writes the json-serialized `predictions`, which are expected to be
    evaluation metrics.
    """

    def __init__(self):
        super().__init__(False)

    @overrides
    def _write(
        self,
        instances: List[InstanceDict],
        predictions: Any,
        output_file_or_dir: str,
        model_name: str,
        *args,
        **kwargs
    ) -> None:
        output_file = output_file_or_dir
        dirname = os.path.dirname(output_file)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        with open(output_file, "w") as out:
            out.write(json.dumps(predictions, indent=2))



import json
import pytest
import unittest
from parameterized import parameterized

from repro.models.chen2020 import LERC, MOCHAEvaluationMetric
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT


class TestChen2020Models(unittest.TestCase):
    def setUp(self) -> None:
        # These examples were taken from the "qaeval" unit tests
        self.examples = json.load(open(f"{FIXTURES_ROOT}/examples.json", "r"))

    @parameterized.expand(get_testing_device_parameters())
    def test_lerc_regression(self, device: int):
        model = LERC(device=device)
        examples = self.examples["lerc"]
        inputs = [
            {
                "context": example["context"],
                "question": example["question"],
                "reference": example["reference"],
                "candidate": example["candidate"],
            }
            for example in examples
        ]
        expected = [example["score"] for example in examples]
        actual = model.predict_batch(inputs)
        assert expected == pytest.approx(actual, abs=1e-4)

    def test_mocha_eval_regression(self):
        metric = MOCHAEvaluationMetric()
        examples = self.examples["evaluation"]
        inputs = examples["inputs"]
        expected = examples["metrics"]
        actual = metric.predict_batch(inputs)
        assert actual == expected



import json
import unittest
from parameterized import parameterized

from repro.models.zhao2019 import MoverScore, MoverScoreForSummarization
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


class TestZhao2019Models(unittest.TestCase):
    def setUp(self) -> None:
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))

    @parameterized.expand(get_testing_device_parameters())
    def test_moverscore_regression(self, device: int):
        model = MoverScore(device=device)
        inputs = [
            {"candidate": inp["candidate"], "references": inp["references"]}
            for inp in self.examples
        ]
        expected_macro = self.expected["default"]["macro"]
        expected_micro = self.expected["default"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(get_testing_device_parameters())
    def test_moverscore_for_summarization_regression(self, device: int):
        model = MoverScoreForSummarization(device=device)
        inputs = [
            {"candidate": inp["candidate"], "references": inp["references"]}
            for inp in self.examples
        ]
        expected_macro = self.expected["summarization"]["macro"]
        expected_micro = self.expected["summarization"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    def test_moverscore_for_summarization_invalid_kwargs(self):
        model = MoverScoreForSummarization()
        with self.assertRaises(Exception):
            model.predict(
                candidate="Candidate", references=["References"], use_stopwords=False
            )

        with self.assertRaises(Exception):
            model.predict_batch([], use_stopwords=False)



import json
import os
from overrides import overrides
from typing import Dict, List

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file
from repro.models import Model, QuestionAnsweringModel


@Model.register("gupta2020-nmn")
class NeuralModuleNetwork(QuestionAnsweringModel):
    def __init__(self, image: str = "gupta2020", device: int = 0) -> None:
        self.image = image
        self.device = device

    @overrides
    def predict_batch(self, inputs: List[Dict[str, str]], *args, **kwargs) -> List[str]:
        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"

            # Serialize the input to a file
            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for inp in inputs:
                    out.write(
                        json.dumps(
                            {"passage": inp["context"], "question": inp["question"]}
                        )
                        + "\n"
                    )

            # Run inference. The output_dir must exist before running
            # the docker command
            os.makedirs(host_output_dir)
            host_output_file = f"{host_output_dir}/output.jsonl"
            container_output_file = f"{container_output_dir}/output.jsonl"

            commands = []
            commands.append("cd nmn-drop")

            # If there is a GPU, we restrict the visible devices to that GPU.
            # Then, the `process_device` is the ID of the GPU for the predict command. After
            # restricting the visible devices to `self.device`, that device now has ID 0.
            cuda = self.device != -1
            if cuda:
                process_device = 0
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
            else:
                process_device = -1

            commands.append(
                f"allennlp predict"
                f"  --output-file {container_output_file}"
                f"  --predictor drop_demo_predictor"
                f"  --include-package semqa"
                f"  --silent"
                f"  --batch-size 1"
                f"  --cuda-device {process_device}"
                f"  ../model.tar.gz"
                f"  {container_input_file}"
            )
            command = " && ".join(commands)

            run_command(
                self.image,
                command,
                volume_map=volume_map,
                network_disabled=True,
                cuda=cuda,
            )

            # The `drop_demo_predictor` does not separate outputs by \n. All
            # of the outputs are on the same line, so pass the `single_line=True`
            # flag to the loader
            results = read_jsonl_file(host_output_file, single_line=True)
            answers = [result["answer"] for result in results]
            return answers



from repro.data.output_writers.output_writer import OutputWriter
from repro.data.output_writers.metrics import MetricsOutputWriter
from repro.data.output_writers.default import DefaultOutputWriter
from repro.data.output_writers.sacrerouge import SacreROUGEOutputWriter



import pytest
import unittest
from parameterized import parameterized

from repro.models.durmus2020 import FEQA
from repro.testing import get_testing_device_parameters


class TestDurmus2020Models(unittest.TestCase):
    @parameterized.expand(get_testing_device_parameters())
    def test_feqa_example(self, device: int):
        # These examples are taken from https://github.com/esdurmus/feqa/blob/master/run_feqa.ipynb
        model = FEQA(device=device)
        inputs = [
            {
                "candidate": "The world's oldest person died in 1898",
                "sources": [
                    "The world's oldest person has died a few weeks after celebrating her 117th birthday. Born on March 5, 1898, the greatgrandmother had lived through two world wars, the invention of the television and the first successful powered aeroplane."
                ],
            },
            {
                "candidate": "The world's oldest person died after her 117th birthday",
                "sources": [
                    "The world's oldest person has died a few weeks after celebrating her 117th birthday. Born on March 5, 1898, the greatgrandmother had lived through two world wars, the invention of the television and the first successful powered aeroplane."
                ],
            },
        ]

        _, actual_micro = model.predict_batch(inputs)
        assert len(actual_micro) == 2
        assert actual_micro[0]["feqa"] == pytest.approx(0.674074074074074, abs=1e-4)
        # This example is slightly off from the expected version value (https://github.com/esdurmus/feqa/blob/master/run_feqa.ipynb)
        # due to the different en_core_web_sm model. The original expected value was 0.8875
        assert actual_micro[1]["feqa"] == pytest.approx(0.85, abs=1e-4)



import json
import logging
import numpy as np
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.data.types import MetricsType, TextType
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("sellam2020-bleurt")
class BLEURT(Model):
    def __init__(
        self,
        image: str = "sellam2020",
        model: str = "bleurt-base-128",
        device: int = 0,
        batch_size: int = 16,
    ):
        self.image = image
        self.model = model
        self.device = device
        self.batch_size = batch_size

    @staticmethod
    def _check_references(references_list: List[List[TextType]]) -> List[TextType]:
        single_references = []
        for references in references_list:
            if len(references) != 1:
                raise Exception(
                    f"BLEURT only supports single references. Found: {len(references)}"
                )
            single_references.append(references[0])
        return single_references

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(
            f"Calculating BLEURT with model {self.model} and "
            f"image {self.image} on {len(inputs)} inputs."
        )

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # BLEURT only supports a single reference. Check to make sure only
        # one has been passed in
        references = self._check_references(references_list)

        # The each candidate and reference must be `str`, not `List[str]`
        candidates = [util.flatten(candidate) for candidate in candidates]
        references = [util.flatten(reference) for reference in references]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, reference in zip(candidates, references):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "reference": reference,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            cuda = self.device != -1
            commands = []
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
            commands.append("cd bleurt")
            commands.append(
                f"python -m bleurt.score_files"
                f"  -sentence_pairs_file {container_input_file}"
                f"  -bleurt_checkpoint ../{self.model}"
                f"  -scores_file {container_output_file}"
                f"  -bleurt_batch_size {self.batch_size}"
            )

            command = " && ".join(commands)
            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=True,
            )

            micro = open(host_output_file, "r").read().splitlines()
            micro = list(map(float, micro))
            macro = np.mean(micro)

            macro = {"bleurt": macro}
            micro = [{"bleurt": score} for score in micro]
            return macro, micro



import argparse
import json
import os
import subprocess
import sys
import tempfile
from typing import List

sys.path.append("PreSumm/src")

from others.utils import clean


def _tokenize(input_file: str) -> List[List[List[str]]]:
    """
    Tokenize the documents in the input file using the Stanford CoreNLP library.
    """
    with tempfile.TemporaryDirectory() as temp:
        # Serialize all of the data to disk, one document per file
        filenames = []
        input_dir = f"{temp}/input"
        os.makedirs(input_dir)
        with open(input_file, "r") as f:
            for i, line in enumerate(f):
                filename = f"{input_dir}/{i}"
                filenames.append(filename)
                with open(filename, "w") as out:
                    out.write(line)

        file_list = f"{temp}/mapping_for_corenlp.txt"
        with open(file_list, "w") as out:
            for filename in filenames:
                out.write(filename + "\n")

        # Run the tokenization
        output_dir = f"{temp}/output"
        os.makedirs(output_dir)
        print("Tokenizing documents with CoreNLP")
        command = [
            "java",
            "edu.stanford.nlp.pipeline.StanfordCoreNLP",
            "-annotators",
            "tokenize,ssplit",
            "-ssplit.newlineIsSentenceBreak",
            "always",
            "-filelist",
            file_list,
            "-outputFormat",
            "json",
            "-outputDirectory",
            output_dir,
        ]
        subprocess.call(command)
        print("Finished tokenizing documents")

        # Load the results
        documents = []
        for i in range(len(filenames)):
            results = json.load(open(f"{output_dir}/{i}.json"))
            sentences = []
            for sentence in results["sentences"]:
                tokens = [token["word"].lower() for token in sentence["tokens"]]
                sentences.append(tokens)

            sentences = [clean(" ".join(sentence)).split() for sentence in sentences]
            documents.append(sentences)

        return documents


def main(args):
    documents = _tokenize(args.input_file)

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)
    with open(args.output_file, "w") as out:
        for document in documents:
            document = " [CLS] [SEP] ".join(
                [" ".join(sentence) for sentence in document]
            )
            out.write(document + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import json
import unittest
from parameterized import parameterized

from repro.models.susanto2016 import RNNTruecaser
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT


class TestSusanto2016(unittest.TestCase):
    def setUp(self) -> None:
        # The expected outputs are the sentences in this file. We will pass the .lower() version
        # to the model
        self.expected_outputs = json.load(open(f"{FIXTURES_ROOT}/expected-output.json"))

    @parameterized.expand(get_testing_device_parameters())
    def test_rnn_truecaser_en(self, device: int):
        model = RNNTruecaser("wiki-truecaser-model-en.tar.gz", device=device)
        inputs = [{"text": text.lower()} for text in self.expected_outputs["en"]]
        predictions = model.predict_batch(inputs)
        assert predictions == self.expected_outputs["en"]

    @parameterized.expand(get_testing_device_parameters())
    def test_rnn_truecaser_es(self, device: int):
        model = RNNTruecaser("wmt-truecaser-model-es.tar.gz", device=device)
        inputs = [{"text": text.lower()} for text in self.expected_outputs["es"]]
        predictions = model.predict_batch(inputs)
        assert predictions == self.expected_outputs["es"]

    @parameterized.expand(get_testing_device_parameters())
    def test_rnn_truecaser_de(self, device: int):
        model = RNNTruecaser("wmt-truecaser-model-de.tar.gz", device=device)
        inputs = [{"text": text.lower()} for text in self.expected_outputs["de"]]
        predictions = model.predict_batch(inputs)
        assert predictions == self.expected_outputs["de"]

    @parameterized.expand(get_testing_device_parameters())
    def test_rnn_truecaser_ru(self, device: int):
        model = RNNTruecaser("lrl-truecaser-model-ru.tar.gz", device=device)
        inputs = [{"text": text.lower()} for text in self.expected_outputs["ru"]]
        predictions = model.predict_batch(inputs)
        assert predictions == self.expected_outputs["ru"]



from overrides import overrides
from typing import List

from repro.common.io import read_jsonl_file
from repro.data.dataset_readers import DatasetReader
from repro.data.types import InstanceDict


@DatasetReader.register("jsonl")
class JSONLinesDatasetReader(DatasetReader):
    @overrides
    def _read(self, *input_files: str) -> List[InstanceDict]:
        """
        Loads the instances from the `input_files`, where each line is expected to be
        a json-serialized object with an "instance_id" key.

        Parameters
        ----------
        input_files : str
            The input files

        Returns
        -------
        List[InstanceDict]
            The instances
        """
        instances = []
        for input_file in input_files:
            instances.extend(read_jsonl_file(input_file))
        return instances



import logging
import os
from overrides import overrides
from typing import Dict, List

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file, write_to_text_file
from repro.models import Model, TruecasingModel

logger = logging.getLogger(__name__)


@Model.register("susanto2016-truecaser")
class RNNTruecaser(TruecasingModel):
    def __init__(
        self,
        model: str,
        image: str = "susanto2016",
        device: int = 0,
    ) -> None:
        """
        Parameters
        ----------
        model : str
            The name of the model, currently either "wiki-truecaser-model-en.tar.gz" for English,
            "wmt-truecaser-model-de.tar.gz" for German, "wmt-truecaser-model-es.tar.gz" for Spanish,
            or "lrl-truecaser-model-ru.tar.gz" for Russian.
        image : str, default="susanto2016"
            The name of the Docker image
        device : int, default=0
            The ID of the GPU, -1 if CPU
        """
        if model not in [
            "lrl-truecaser-model-ru.tar.gz",
            "wiki-truecaser-model-en.tar.gz",
            "wmt-truecaser-model-de.tar.gz",
            "wmt-truecaser-model-es.tar.gz",
        ]:
            raise Exception(f"Unknown model: {model}")

        self.model = model
        self.image = image
        self.device = device

    @overrides
    def predict_batch(self, inputs: List[Dict[str, str]], *args, **kwargs) -> List[str]:
        input_texts = [inp["text"] for inp in inputs]
        logger.info(f"Running truecasing on {len(inputs)} using image {self.image}")

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.txt"
            container_input_file = f"{container_input_dir}/input.txt"
            write_to_text_file(input_texts, host_input_file)

            # Run inference. The output_dir must exist before running
            # the docker command
            os.makedirs(host_output_dir)
            host_output_file = f"{host_output_dir}/output.txt"
            container_output_file = f"{container_output_dir}/output.txt"

            commands = []
            commands.append("cd pytorch-truecaser")

            cuda = self.device != -1
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
                process_device = 0
            else:
                process_device = -1

            commands.append(
                f"allennlp predict"
                f"  ../{self.model}"
                f"  {container_input_file}"
                f"  --output-file {container_output_file}"
                f"  --include-package mylib "
                f"  --use-dataset-reader "
                f"  --predictor truecaser-predictor"
                f"  --cuda-device {process_device}"
                f"  --silent"
            )

            command = " && ".join(commands)
            run_command(
                self.image,
                command,
                volume_map=volume_map,
                network_disabled=True,
                cuda=cuda,
            )

            outputs = open(host_output_file, "r").read().splitlines()
            return outputs



from typing import Dict, List, T, Tuple, Union

from repro.data.types import TextType

# `Indexable` is something which maps from an int to a value.
Indexable = Union[List[T], Dict[int, T]]

NestedDict = Union[Dict[str, float], "NestedDict"]


def flatten(text: Union[str, List[str]], separator: str = None) -> str:
    """
    Flattens the text item to a string. If the input is a string, that
    same string is returned. Otherwise, the text is joined together with
    the separator.

    Parameters
    ----------
    text : Union[str, List[str]]
        The text to flatten
    separator : str, default=None
        The separator to join the list with. If `None`, the separator will be " "

    Returns
    -------
    str
        The flattened text
    """
    separator = separator or " "
    if isinstance(text, list):
        return separator.join(text)
    return text


def _hash_texts(texts: List[TextType]) -> int:
    hashes = []
    for text in texts:
        if isinstance(text, str):
            hashes.append(hash(text))
        else:
            hashes.append(hash(tuple(text)))
    return hash(tuple(hashes))


def group_by_references(
    candidates: List[TextType], references_list: List[List[TextType]]
) -> Tuple[List[List[TextType]], List[List[TextType]], List[Tuple[int, int]]]:
    """
    Groups the `candidates` by identical references in `references_list`. This is
    enables evaluation metrics which need to do a lot of preprocessing of the
    references to reducing the amount of duplicate effort required.

    Parameters
    ----------
    candidates : List[TextType]
        The candidate texts
    references_list : List[List[TextType]]
        The reference texts.

    Returns
    -------
    List[List[TextType]]
        The grouped candidate texts in which the candidates at index `i` all
        share the same references in the grouped references
    List[List[TextType]]
        The grouped reference texts
    List[Tuple[int, int]]
        A mapping from the input `candidates` to the `(i, j)` position
        in the grouped candidates that it corresponds to.
    """
    mapping = []
    references_to_index = {}
    grouped_candidates_list = []
    grouped_references_list = []

    for candidate, references in zip(candidates, references_list):
        references_key = _hash_texts(references)
        if references_key not in references_to_index:
            # This is a new set of references. Update the data structures
            group_index = len(grouped_references_list)
            references_to_index[references_key] = group_index
            grouped_candidates_list.append([])
            grouped_references_list.append(references)

        group_index = references_to_index[references_key]
        candidate_index = len(grouped_candidates_list[group_index])
        grouped_candidates_list[group_index].append(candidate)
        mapping.append((group_index, candidate_index))

    return grouped_candidates_list, grouped_references_list, mapping


def ungroup_values(
    values: Indexable[Indexable[T]], mapping: List[Tuple[int, int]]
) -> List[T]:
    """
    Ungroups the values in `values_list` based on the group `mapping`. `mapping[i]` specifies
    the `(j, k)` position in `values_list` that the value for `i` corresponds to. The
    result of the function is the ungrouped values for each item in `mapping`.

    Parameters
    ----------
    values : Indexable[Indexable[T]]
        The grouped values
    mapping : List[Tuple[int, int]]
        The mapping which specifies where in the grouped values each
        item in the original order is in.

    Returns
    -------
    List[T]
        The ungrouped values.
    """
    ungrouped_values = []
    for i, j in mapping:
        ungrouped_values.append(values[i][j])
    return ungrouped_values


def flatten_nested_dict(nested_dict: NestedDict) -> Dict[Tuple[str, ...], float]:
    def _recursively_flatten(
        target: Dict[Tuple[str, ...], float], d: NestedDict, prefix: List[str]
    ) -> Dict[Tuple[str, ...], float]:
        for key, value in d.items():
            if isinstance(value, dict):
                _recursively_flatten(target, value, prefix + [key])
            else:
                target[tuple(prefix + [key])] = value
        return target

    return _recursively_flatten({}, nested_dict, [])


def unflatten_dict(d: Dict[Tuple[str, ...], float]) -> NestedDict:
    def _recursively_set(
        target: NestedDict, keys: Tuple[str], value: float, index: int
    ) -> NestedDict:
        if index == len(keys) - 1:
            target[keys[index]] = value
        else:
            key = keys[index]
            if key in target:
                _recursively_set(target[key], keys, value, index + 1)
            else:
                target[key] = _recursively_set({}, keys, value, index + 1)
        return target

    unflattened = {}
    for keys, value in d.items():
        _recursively_set(unflattened, keys, value, 0)
    return unflattened


def average_dicts(dicts: List[NestedDict]) -> NestedDict:
    if len(dicts) == 0:
        return {}

    # First, flatten all the dicts to make processing eaiser
    flattened_dicts = [flatten_nested_dict(d) for d in dicts]

    # Ensure that they all have the same keys. Different keys is not
    # currently supported
    keys = flattened_dicts[0].keys()
    for d in flattened_dicts[1:]:
        if d.keys() != keys:
            raise Exception(
                f"Dicts do not have identical keys. Expected {keys}, found {d.keys()}"
            )

    # Sum the values
    aggregate = {}
    for d in flattened_dicts:
        for key in keys:
            if key not in aggregate:
                aggregate[key] = d[key]
            else:
                aggregate[key] += d[key]

    # Normalize the values to calculate the average
    average = {key: value / len(dicts) for key, value in aggregate.items()}

    # Unflatten the average to match the input dicts
    return unflatten_dict(average)



import argparse

from overrides import overrides

from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand, build_image


@SetupSubcommand.register("liu2019")
class Liu2019SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("liu2019", f"{MODELS_ROOT}/liu2019")

    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = f'Build the docker image "{self.image}"'
        self.parser = parser.add_parser(
            self.image, description=description, help=description
        )
        self.parser.add_argument(
            "--transformerabs-cnndm",
            action="store_true",
            help="Indicates the TransformerAbs-CNNDM model should be downloaded",
        )
        self.parser.add_argument(
            "--bertsumext-cnndm",
            action="store_true",
            help="Indicates the BertSumExt-CNNDM model should be downloaded",
        )
        self.parser.add_argument(
            "--bertsumextabs-cnndm",
            action="store_true",
            help="Indicates the BertSumExtAbs-CNNDM model should be downloaded",
        )
        self.parser.add_argument(
            "--bertsumextabs-xsum",
            action="store_true",
            help="Indicates the BertSumExtAbs-XSum model should be downloaded",
        )
        self.parser.add_argument(
            "--silent",
            action="store_true",
            help="Silences the output from the build command",
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        build_args = {
            "TRANSFORMERABS_CNNDM": "true" if args.transformerabs_cnndm else "false",
            "BERTSUMEXT_CNNDM": "true" if args.bertsumext_cnndm else "false",
            "BERTSUMEXTABS_CNNDM": "true" if args.bertsumextabs_cnndm else "false",
            "BERTSUMEXTABS_XSUM": "true" if args.bertsumextabs_xsum else "false",
        }
        build_image(self.root, self.image, build_args=build_args, silent=args.silent)



import argparse

from overrides import overrides

from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand, build_image


@SetupSubcommand.register("sellam2020")
class Sellam20SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("sellam2020", f"{MODELS_ROOT}/sellam2020")

    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = f'Build the docker image "{self.image}"'
        self.parser = parser.add_parser(
            self.image, description=description, help=description
        )
        # The default models
        self.parser.add_argument(
            "--not-tiny-128",
            action="store_true",
            help="Indicates the BLEURT-Tiny-128 model should *not* be downloaded",
        )
        self.parser.add_argument(
            "--not-base-128",
            action="store_true",
            help="Indicates the BLEURT-Base-128 model should *not* be downloaded",
        )

        # Optional models
        self.parser.add_argument(
            "--tiny-512",
            action="store_true",
            help="Indicates the BLEURT-Tiny-512 model should be downloaded",
        )
        self.parser.add_argument(
            "--base-512",
            action="store_true",
            help="Indicates the BLEURT-Base-512 model should be downloaded",
        )
        self.parser.add_argument(
            "--large-128",
            action="store_true",
            help="Indicates the BLEURT-Large-128 model should be downloaded",
        )
        self.parser.add_argument(
            "--large-512",
            action="store_true",
            help="Indicates the BLEURT-Large-512 model should be downloaded",
        )

        self.parser.add_argument(
            "--silent",
            action="store_true",
            help="Silences the output from the build command",
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        build_args = {
            "TINY_128": "false" if args.not_tiny_128 else "true",
            "BASE_128": "false" if args.not_base_128 else "true",
            "TINY_512": "true" if args.tiny_512 else "false",
            "BASE_512": "true" if args.base_512 else "false",
            "LARGE_128": "true" if args.large_128 else "false",
            "LARGE_512": "true" if args.large_512 else "false",
        }
        build_image(self.root, self.image, build_args=build_args, silent=args.silent)



import datasets
import logging
import os
from datasets.config import HF_DATASETS_CACHE
from overrides import overrides
from typing import List

from repro.data.dataset_readers import DatasetReader
from repro.data.types import InstanceDict

DEFAULT_VERSIONS = {"cnn_dailymail": "3.0.0", "xsum": "1.2.0"}

logger = logging.getLogger(__name__)


def hf_dataset_exists_locally(name: str, version: str = None) -> bool:
    """
    Checks to see if a Huggingface `datasets` dataset exists locally in the cache.
    The logic checks to see if the directory exists where the data should be, but
    does not do any further verification.

    Parameters
    ----------
    name : str
        The name of the dataset, like "cnn_dailymail"
    version : str, default=None
        The version of the dataset, like "3.0.0". If `None`, then the
        default version is used if one exists.

    Returns
    -------
    bool
        True if the dataset exists, False otherwise.
    """
    if version is None:
        version = DEFAULT_VERSIONS[name]
    data_dir = os.path.join(HF_DATASETS_CACHE, name, version)
    return os.path.exists(data_dir)


@DatasetReader.register("huggingface-datasets")
class HuggingfaceDatasetsDatasetReader(DatasetReader):
    def __init__(self, dataset_name: str, split: str):
        self.dataset_name = dataset_name
        self.split = split

    @overrides
    def _read(self) -> List[InstanceDict]:
        # Split the version if it exists, taking the default if not
        parts = self.dataset_name.split("/")
        if len(parts) == 1:
            name = self.dataset_name
            if name not in DEFAULT_VERSIONS:
                logger.warning(
                    f"Unknown default dataset version for dataset: {name}. Using `None`"
                )
                version = None
            else:
                version = DEFAULT_VERSIONS[name]
        elif len(parts) == 2:
            name = parts[0]
            version = parts[1]
        else:
            raise Exception(
                f"Unknown dataset format. Expected 0 or 1 '/': {self.dataset_name}"
            )

        dataset = datasets.load_dataset(name, version, split=self.split)

        instances = []
        if name == "cnn_dailymail":
            for instance in dataset:
                # The reference sentences are separated by \n, but the document
                # is not sentence split already
                reference = instance["highlights"].split("\n")
                instances.append(
                    {
                        "instance_id": instance["id"],
                        "document": instance["article"],
                        "reference": reference,
                    }
                )
        elif name == "mocha":
            for instance in dataset:
                instances.append(
                    {
                        "instance_id": instance["id"],
                        "constituent_dataset": instance["constituent_dataset"],
                        "context": instance["context"],
                        "question": instance["question"],
                        "reference": instance["reference"],
                        "candidate": instance["candidate"],
                        "score": instance["score"],
                        "metadata": instance["metadata"],
                    }
                )
        elif name == "scientific_papers":
            for i, instance in enumerate(dataset):
                # There is no instance_id in `datasets`, so we make one up
                instance_id = f"{version}-{self.split}-{i}"

                # The articles and abstracts have paragraphs split by \n
                document = instance["article"].split("\n")
                summary = instance["abstract"].split("\n")

                # The section names are split by \n
                section_names = instance["section_names"].split("\n")

                instances.append(
                    {
                        "instance_id": instance_id,
                        "document": document,
                        "reference": summary,
                        "section_names": section_names,
                    }
                )
        elif name == "squad_v2":
            for instance in dataset:
                instances.append(
                    {
                        "instance_id": instance["id"],
                        "context": instance["context"],
                        "question": instance["question"],
                        "answers": instance["answers"],
                    }
                )
        elif name == "xsum":
            for instance in dataset:
                # The documents include \n characters to separate sentences. The
                # summaries are only one sentence, so it does not matter for them
                document = instance["document"].split("\n")
                instances.append(
                    {
                        "instance_id": instance["id"],
                        "document": document,
                        "reference": instance["summary"],
                    }
                )
        else:
            raise Exception(f"Unsupported dataset: {self.dataset_name}")

        return instances



import json
import unittest
from parameterized import parameterized

from repro.models.sellam2020 import BLEURT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


class TestSellam2020Models(unittest.TestCase):
    def setUp(self) -> None:
        self.bleurt_examples = json.load(
            open(f"{FIXTURES_ROOT}/bleurt-unittests.json", "r")
        )

    @parameterized.expand(get_testing_device_parameters())
    def test_bleurt_unittest_examples(self, device: int):
        # Tests the examples from the BLEURT repository unit tests
        model = BLEURT(model="bleurt/bleurt/test_checkpoint", device=device)
        inputs = self.bleurt_examples["inputs"]
        expected_macro = self.bleurt_examples["output"]["macro"]
        expected_micro = self.bleurt_examples["output"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)



import argparse
import json

from feqa import FEQA


def main(args):
    cuda = args.cuda_device != -1
    metric = FEQA(use_gpu=cuda)
    metric.batch_size = args.batch_size

    candidates = []
    sources = []
    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            candidates.append(data["candidate"])
            sources.append(data["source"])

    scores = metric.compute_score(sources, candidates, aggregate=False)

    with open(args.output_file, "w") as out:
        for score in scores:
            out.write(json.dumps({"feqa": score}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--cuda-device", required=True, type=int)
    argp.add_argument("--batch-size", required=True, type=int)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import json
import os
from typing import List, Union

from repro.common import util


def write_to_text_file(
    items: List[Union[str, List[str]]], file_path: str, separator: str = None
) -> None:
    """
    Writes the items in `items` to a text file with one item per line. If an individual
    item is a list, it is first flattened with the `separator`.

    Parameters
    ----------
    items : List[Union[str, List[str]]]
        The items to write to a file.
    file_path : str
        The path to the file where the output should be written
    separator : str, default=None
        The separator to use to join the items
    """
    dirname = os.path.dirname(file_path)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(file_path, "w") as out:
        for item in items:
            item = util.flatten(item, separator=separator)
            out.write(item + "\n")


def write_to_jsonl_file(
    items: List[Union[str, List[str]]],
    key: str,
    file_path: str,
    flatten: bool = False,
    separator: str = None,
) -> None:
    """
    Writes the items in `items` to a jsonl file at `file_path`. Each item will correspond to one line. The
    item will be the value for the `key` in each json. If `flatten` is `True`, each item will
    first be flattened using `separator`.

    Parameters
    ----------
    items : List[Union[str, List[str]]]
        The items to serialize
    key : str
        The key for the items
    file_path : str
        The path to the file where the output should be written
    flatten : bool, default=False
        Indicates whether the items should first be flattened
    separator : str, default=None
        The separator for flattening the items
    """
    dirname = os.path.dirname(file_path)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(file_path, "w") as out:
        for item in items:
            if flatten:
                item = util.flatten(item, separator=separator)
            out.write(json.dumps({key: item}) + "\n")


def read_jsonl_file(file_path: str, single_line: bool = False) -> List:
    """
    Loads a jsonl file and returns a list corresponding to each line in the file.

    Parameters
    ----------
    file_path : str
        The file to load the data from
    single_line : bool, default=False
        Indicates all of the json objects are on a single line (i.e., not separated
        by a newline character, but separated by "")

    Returns
    -------
    List
        The items, where each item corresponds to one line
    """
    items = []
    if single_line:
        decoder = json.JSONDecoder()
        contents = open(file_path, "r").read()
        offset = 0
        while offset < len(contents):
            item, length = decoder.raw_decode(contents[offset:])
            items.append(item)
            offset += length
    else:
        with open(file_path, "r") as f:
            for line in f:
                items.append(json.loads(line))
    return items



import json
import logging
import os
from typing import Any, Dict, List, Union

from overrides import overrides

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("chen2020-lerc")
class LERC(Model):
    def __init__(
        self, image: str = "chen2020", device: int = 0, batch_size: int = 8
    ) -> None:
        self.image = image
        self.device = device
        self.batch_size = batch_size
        self.model = "lerc-2020-11-18.tar.gz"

    @overrides
    def predict(
        self, context: str, question: str, reference: str, candidate: str, **kwargs
    ) -> float:
        return self.predict_batch(
            [
                {
                    "context": context,
                    "question": question,
                    "reference": reference,
                    "candidate": candidate,
                }
            ],
            **kwargs,
        )[0]

    @overrides
    def predict_batch(self, inputs: List[Dict[str, Any]], **kwargs) -> List[float]:
        logger.info(
            f"Predicting scores for {len(inputs)} inputs, image {self.image}, and model {self.model}"
        )

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            host_output_file = f"{host_output_dir}/output.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"
            container_output_file = f"{container_output_dir}/output.jsonl"

            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for inp in inputs:
                    out.write(
                        json.dumps(
                            {
                                "context": inp["context"],
                                "question": inp["question"],
                                "reference": inp["reference"],
                                "candidate": inp["candidate"],
                            }
                        )
                        + "\n"
                    )

            cuda = self.device != -1
            predict_device = 0 if cuda else -1
            commands = ["cd MOCHA"]
            predict_command = (
                f"python predict.py"
                f"  --input-file {container_input_file}"
                f"  --model-path ../{self.model}"
                f"  --batch-size {self.batch_size}"
                f"  --cuda-device {predict_device}"
                f"  --output-file {container_output_file}"
            )
            if cuda:
                predict_command = (
                    f"CUDA_VISIBLE_DEVICES={self.device} " + predict_command
                )
            commands.append(predict_command)

            command = " && ".join(commands)
            os.makedirs(host_output_dir)
            run_command(
                self.image,
                command,
                volume_map=volume_map,
                cuda=cuda,
                network_disabled=False,
            )

            outputs = read_jsonl_file(host_output_file)
            scores = [output["pred_score"] for output in outputs]
            return scores


@Model.register("chen2020-eval")
class MOCHAEvaluationMetric(Model):
    def __init__(self, image: str = "chen2020") -> None:
        self.image = image

    @overrides
    def predict(
        self,
        dataset: str,
        source: str,
        score: float,
        prediction: float,
        **kwargs,
    ) -> Dict[str, float]:
        # You can't compute this metric with just one example. We define
        # this method so the "predict" command can identify the required arguments.
        raise NotImplementedError

    @overrides
    def predict_batch(
        self, inputs: List[Dict[str, Union[str, float]]], **kwargs
    ) -> Dict[str, float]:
        logger.info(f"Evaluating {len(inputs)} inputs")
        with TemporaryDirectory() as temp:
            # The output file is hard-coded to be relative to the prediction file,
            # so we don't need an output directory
            host_input_dir = f"{temp}/input"
            volume_map = make_volume_map(host_input_dir)
            container_input_dir = volume_map[host_input_dir]

            host_annotations_file = f"{host_input_dir}/annotations.jsonl"
            host_predictions_file = f"{host_input_dir}/predictions.jsonl"
            host_output_file = f"{host_input_dir}/predictions.jsonl.corrs"
            container_annotations_file = f"{container_input_dir}/annotations.jsonl"
            container_predictions_file = f"{container_input_dir}/predictions.jsonl"

            annotations = {}
            predictions = {}
            for i, inp in enumerate(inputs):
                dataset = inp["dataset"]
                if dataset not in annotations:
                    annotations[dataset] = {}
                    predictions[dataset] = {}

                instance_id = str(i)
                annotations[dataset][instance_id] = {
                    "score": inp["score"],
                    "metadata": {"source": inp["source"]},
                }
                predictions[dataset][instance_id] = {
                    "pred_score": inp["prediction"],
                    "metadata": {"source": inp["source"]},
                }

            os.makedirs(host_input_dir)
            with open(host_annotations_file, "w") as out:
                out.write(json.dumps(annotations))
            with open(host_predictions_file, "w") as out:
                out.write(json.dumps(predictions))

            command = (
                f"cd MOCHA && "
                f"python evaluate_mocha_preds.py"
                f"  --annotations {container_annotations_file}"
                f"  --predictions {container_predictions_file}"
            )
            run_command(
                self.image, command, volume_map=volume_map, network_disabled=True
            )

            metrics = json.load(open(host_output_file, "r"))
            return metrics



from ._models.dou2021.src.dataset_reader import Dou2021DatasetReader
from ._models.dou2021.src.models import OracleSentenceGSumModel, SentenceGSumModel
from ._models.dou2021.src.setup import Dou2021SetupSubcommand



import json
import logging
from typing import Any, Dict, List, Tuple, Union

from overrides import overrides

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model, QuestionAnsweringModel, QuestionGenerationModel

QAPairsType = List[List[Dict[str, Any]]]

logger = logging.getLogger(__name__)


@Model.register("deutsch2021-qaeval")
class QAEval(Model):
    def __init__(
        self,
        image: str = "deutsch2021",
        device: int = 0,
        generation_batch_size: int = 8,
        answering_batch_size: int = 8,
        lerc_batch_size: int = 8,
    ):
        self.image = image
        self.device = device
        self.generation_batch_size = generation_batch_size
        self.answering_batch_size = answering_batch_size
        self.lerc_batch_size = lerc_batch_size

    def predict(
        self, candidate: TextType, references: List[TextType], **kwargs
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self,
        inputs: List[Dict[str, Union[str, List[str]]]],
        return_qa_pairs=False,
        **kwargs,
    ) -> Union[
        Tuple[MetricsType, List[MetricsType]],
        Tuple[MetricsType, List[MetricsType], List[QAPairsType]],
    ]:
        logger.info(f"Calculating QAEval for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, references in zip(candidates, references_list):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "references": references,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            commands = []
            cuda = self.device != -1
            if cuda:
                predict_device = 0
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
            else:
                predict_device = -1

            kwargs = {
                "cuda_device": predict_device,
                "generation_batch_size": self.generation_batch_size,
                "answering_batch_size": self.answering_batch_size,
                "use_lerc": True,
                "lerc_batch_size": self.lerc_batch_size,
            }
            kwargs_str = json.dumps(kwargs)
            commands.append(
                f"python score.py"
                f"  --input-file {container_input_file}"
                f"  --kwargs '{kwargs_str}'"
                f"  --output-file {container_output_file}"
            )

            command = " && ".join(commands)
            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=True,
            )

            results = read_jsonl_file(host_output_file)
            micro_metrics = [result["metrics"] for result in results]
            macro_metrics = util.average_dicts(micro_metrics)

            if return_qa_pairs:
                qa_pairs = [result["qa_pairs"] for result in results]
                return macro_metrics, micro_metrics, qa_pairs
            else:
                return macro_metrics, micro_metrics


@Model.register("deutsch2021-question-generation")
class QAEvalQuestionGenerationModel(QuestionGenerationModel):
    def __init__(
        self, image: str = "deutsch2021", device: int = 0, batch_size: int = 8
    ) -> None:
        self.image = image
        self.device = device
        self.batch_size = batch_size

    @overrides
    def predict_batch(self, inputs: List[Dict[str, str]], **kwargs) -> List[str]:
        logger.info(f"Generating questions for {len(inputs)} inputs")

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"
            with open(host_input_file, "w") as out:
                for inp in inputs:
                    out.write(
                        json.dumps(
                            {
                                "context": inp["context"],
                                "start": inp["start"],
                                "end": inp["end"],
                            }
                        )
                        + "\n"
                    )

            cuda = self.device != -1
            predict_device = 0 if cuda else -1
            command = (
                f"python generate_questions.py"
                f"  --input-file {container_input_file}"
                f"  --model-file models/generation/model.tar.gz"
                f"  --cuda-device {predict_device}"
                f"  --batch-size {self.batch_size}"
                f"  --output-file {container_output_file}"
            )
            if cuda:
                command = f"CUDA_VISIBLE_DEVICES={self.device} " + command

            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=True,
            )

            questions = [
                output["question"] for output in read_jsonl_file(host_output_file)
            ]
            return questions


@Model.register("deutsch2021-question-answering")
class QAEvalQuestionAnsweringModel(QuestionAnsweringModel):
    def __init__(
        self, image: str = "deutsch2021", device: int = 0, batch_size: int = 8
    ) -> None:
        self.image = image
        self.device = device
        self.batch_size = batch_size

    @overrides
    def predict_batch(
        self, inputs: List[Dict[str, str]], return_dicts: bool = False, **kwargs
    ) -> List[str]:
        logger.info(f"Answering for {len(inputs)} inputs")

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"
            with open(host_input_file, "w") as out:
                for inp in inputs:
                    out.write(
                        json.dumps(
                            {
                                "context": inp["context"],
                                "question": inp["question"],
                            }
                        )
                        + "\n"
                    )

            cuda = self.device != -1
            predict_device = 0 if cuda else -1
            command = (
                f"python answer_questions.py"
                f"  --input-file {container_input_file}"
                f"  --model-dir models/answering"
                f"  --cuda-device {predict_device}"
                f"  --batch-size {self.batch_size}"
                f"  --output-file {container_output_file}"
            )
            if cuda:
                command = f"CUDA_VISIBLE_DEVICES={self.device} " + command

            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=True,
            )

            outputs = read_jsonl_file(host_output_file)
            if not return_dicts:
                outputs = [
                    output["prediction"]
                    if output["probability"] > output["null_probability"]
                    else None
                    for output in outputs
                ]
            return outputs



import argparse
import os
from nltk.tokenize import sent_tokenize
from tqdm import tqdm


def main(args):
    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in tqdm(f, desc="Running sentence splitting"):
                line = line.strip()
                sentences = [sentence for sentence in sent_tokenize(line)]
                out.write("<q>".join(sentences) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("input_file")
    argp.add_argument("output_file")
    args = argp.parse_args()
    main(args)



import argparse
import json
import os

from qaeval import QuestionAnsweringModel


def main(args):
    inputs = []
    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            inputs.append((data["question"], data["context"]))

    model = QuestionAnsweringModel(args.model_dir, args.cuda_device, args.batch_size)
    answers_dicts = model.answer_all(
        inputs, return_offsets=True, try_fixing_offsets=True, return_dicts=True
    )

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, "w") as out:
        for answers_dict in answers_dicts:
            out.write(
                json.dumps(
                    {
                        "prediction": answers_dict["prediction"],
                        "probability": answers_dict["probability"],
                        "null_probability": answers_dict["null_probability"],
                        "start": answers_dict["start"],
                        "end": answers_dict["end"],
                    }
                )
                + "\n"
            )


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--model-dir", required=True)
    argp.add_argument("--cuda-device", type=int, required=True)
    argp.add_argument("--batch-size", type=int, required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import json
import logging
import os
from typing import Dict, List, Union

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.data.types import MetricsType
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("squad-v2")
class SQuADv2Evaluation(Model):
    def __init__(self, image: str = "squad-v2"):
        self.image = image

    def predict(
        self, instance_id: str, prediction: str, null_probability: float, **kwargs
    ) -> MetricsType:
        # We cannot evaluate just one instance
        raise NotImplementedError

    def predict_batch(
        self, inputs: List[Dict[str, Union[str, float]]], *args, **kwargs
    ) -> MetricsType:
        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_pred_file = f"{host_input_dir}/pred.json"
            host_na_prob_file = f"{host_input_dir}/na_prob.json"
            container_pred_file = f"{container_input_dir}/pred.json"
            container_na_probfile = f"{container_input_dir}/na_prob.json"

            predictions = {}
            na_probs = {}
            for inp in inputs:
                instance_id = inp["instance_id"]
                predictions[instance_id] = inp["prediction"]
                na_probs[instance_id] = inp["null_probability"]

            os.makedirs(host_input_dir)
            with open(host_pred_file, "w") as out:
                out.write(json.dumps(predictions, indent=2))

            with open(host_na_prob_file, "w") as out:
                out.write(json.dumps(na_probs, indent=2))

            host_output_file = f"{host_output_dir}/eval.json"
            container_output_file = f"{container_output_dir}/eval.json"

            command = (
                f"python evaluate-v2.0.py"
                f"  dev-v2.0.json"
                f"  {container_pred_file}"
                f"  --na-prob-file {container_na_probfile}"
                f"  --out-file {container_output_file}"
            )
            os.makedirs(host_output_dir)
            run_command(
                self.image, command, volume_map=volume_map, network_disabled=True
            )

            metrics = json.load(open(host_output_file, "r"))
            return metrics



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand


@SetupSubcommand.register("gupta2020")
class Gupta2020SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("gupta2020", f"{MODELS_ROOT}/gupta2020")



import shutil
import tempfile
from typing import Optional


class TemporaryDirectory(object):
    """
    Creates a temporary directory that works with a context manager (the python
    ``with`` statement). This class was created because the user is responsible for
    deleting the directory created by ``tempfile.mkdtemp``. Instead, the context
    manager ensures the directory is deleted at the end.

    Example usage::

        with TemporaryDirectory() as temp_dir:
            with open(temp_dir + '/file.txt') as out:
                ...

    Parameters
    ----------
    root: ``str``, optional (default = ``None``)
        The root directory where the temporary directory should be created. If ``None``,
        the ``tempfile.mkdtemp`` default location is used.
    persist: ``bool``, optional (default = False)
        Indicates whether or not the directory should be persist on disk after the
        context closes.
    """

    def __init__(self, root: Optional[str] = None, persist: bool = False) -> None:
        self.root = root
        self.persist = persist

    def __enter__(self):
        self.path = tempfile.mkdtemp(dir=self.root)
        return self.path

    def __exit__(self, *args):
        if not self.persist:
            shutil.rmtree(self.path)



import json
import unittest
from parameterized import parameterized

from repro.models.dou2021 import OracleSentenceGSumModel, SentenceGSumModel
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT
from ..src.commands import get_oracle_sentences, generate_summaries, sentence_split


class TestDou2021Models(unittest.TestCase):
    def setUp(self) -> None:
        self.image = "dou2021"
        # The documents and references were taken from https://github.com/icml-2020-nlp/semsim/tree/master/datasets.
        # The guidance field was taken from the provided outputs of the authors (except for the second example
        # in which the sentences were in a different order). The other fields are regression outputs
        # (i.e., we ran the code on those documents and references and saved the outputs).
        self.examples = json.load(open(f"{FIXTURES_ROOT}/examples.json", "r"))

    def test_sentence_split_regression(self):
        documents = [inp["document"] for inp in self.examples]
        expected = [inp["document_sentences"] for inp in self.examples]
        actual = sentence_split(self.image, documents)
        assert expected == actual

        references = [inp["reference"] for inp in self.examples]
        expected = [inp["reference_sentences"] for inp in self.examples]
        actual = sentence_split(self.image, references)
        assert expected == actual

    def test_get_oracle_sentences(self):
        documents = [inp["document_sentences"] for inp in self.examples]
        references = [inp["reference_sentences"] for inp in self.examples]

        expected = [inp["guidance"] for inp in self.examples]
        guidance = get_oracle_sentences(self.image, documents, references)
        assert guidance == expected

    @parameterized.expand(get_testing_device_parameters())
    def test_generate_summaries(self, device: int):
        model = "bart_sentence"
        batch_size = 16

        documents = [inp["document"] for inp in self.examples]
        guidance = [inp["guidance"] for inp in self.examples]
        expected = [inp["oracle_guided_summary"] for inp in self.examples]
        actual = generate_summaries(
            self.image, model, device, batch_size, documents, guidance
        )
        assert expected == actual

    @parameterized.expand(get_testing_device_parameters())
    def test_oracle_sentence_guided(self, device: int):
        model = OracleSentenceGSumModel(device=device)

        # First without pre-sentence-split documents and references
        inputs = [
            {"document": example["document"], "reference": example["reference"]}
            for example in self.examples
        ]
        expected_summaries = [data["oracle_guided_summary"] for data in self.examples]
        summaries = model.predict_batch(inputs)
        assert summaries == expected_summaries

        # Now with pre-tokenized documents and references
        inputs = [
            {
                "document": example["document_sentences"],
                "reference": example["reference_sentences"],
            }
            for example in self.examples
        ]
        expected_summaries = [
            data["oracle_guided_summary_presplit"] for data in self.examples
        ]
        summaries = model.predict_batch(inputs)
        assert summaries == expected_summaries

    @parameterized.expand(get_testing_device_parameters())
    def test_oracle_sentence_guided_with_guidance(self, device: int):
        model = OracleSentenceGSumModel(device=device)

        # First without pre-sentence-split documents and references
        inputs = [
            {"document": example["document"], "guidance": " ".join(example["guidance"])}
            for example in self.examples
        ]
        expected_summaries = [data["oracle_guided_summary"] for data in self.examples]
        summaries = model.predict_batch(inputs)
        assert summaries == expected_summaries

        # Now with pre-tokenized documents and guidance
        inputs = [
            {
                "document": example["document_sentences"],
                "guidance": example["guidance"],
            }
            for example in self.examples
        ]
        expected_summaries = [
            data["oracle_guided_summary_presplit"] for data in self.examples
        ]
        summaries = model.predict_batch(inputs)
        assert summaries == expected_summaries

    @parameterized.expand(get_testing_device_parameters())
    def test_sentence_guided(self, device: int):
        model = SentenceGSumModel(device=device)

        # Original document text
        inputs = [{"document": example["document"]} for example in self.examples]
        expected_summaries = [data["guided_summary"] for data in self.examples]
        summaries = model.predict_batch(inputs)
        assert summaries == expected_summaries

        # Pre-sentence-split documents
        inputs = [
            {"document": example["document_sentences"]} for example in self.examples
        ]
        expected_summaries = [data["guided_summary_presplit"] for data in self.examples]
        summaries = model.predict_batch(inputs)
        assert summaries == expected_summaries



import argparse
import json
from tqdm import tqdm

from questeval.questeval_metric import QuestEval


def main(args):
    kwargs = json.loads(args.kwargs)
    is_cuda = kwargs.pop("isCuda", args.cuda_device != -1)
    metric = QuestEval(isCuda=is_cuda, **kwargs)

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in tqdm(f, desc="Scoring"):
                data = json.loads(line)
                candidate = data["candidate"]
                source = data["source"]
                reference = data["reference"]

                scores = metric.compute_all(
                    candidate, source=source, reference=reference
                )
                out.write(json.dumps(scores) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--kwargs", required=True)
    argp.add_argument("--cuda-device", required=True, type=int)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



from overrides import overrides
from typing import List

from repro.data.dataset_readers import DatasetReader
from repro.data.types import InstanceDict


@DatasetReader.register("dou2021")
class Dou2021DatasetReader(DatasetReader):
    @overrides
    def _read(
        self, source_file: str, target_file: str, guidance_file: str = None
    ) -> List[InstanceDict]:
        source = open(source_file, "r").read().splitlines()
        target = open(target_file, "r").read().splitlines()
        guidance = (
            open(guidance_file, "r").read().splitlines()
            if guidance_file is not None
            else None
        )

        instances = []
        for i, (document, reference) in enumerate(zip(source, target)):
            instance = {
                "instance_id": str(i),
                "document": document,
                "reference": reference,
            }
            if guidance is not None:
                instance["guidance"] = guidance[i]
            instances.append(instance)
        return instances



import argparse
import importlib
import os
import pkgutil
import sys
from contextlib import contextmanager
from pathlib import Path
from typing import Generator, Optional, Set, T, Union

from repro.commands.subcommand import RootSubcommand
from repro.common import Registrable

PathType = Union[os.PathLike, str]
ContextManagerFunctionReturnType = Generator[T, None, None]


@contextmanager
def push_python_path(path: PathType) -> ContextManagerFunctionReturnType[None]:
    """
    Prepends the given path to `sys.path`.
    This method is intended to use with `with`, so after its usage, its value willbe removed from
    `sys.path`.
    """
    # In some environments, such as TC, it fails when sys.path contains a relative path, such as ".".
    path = Path(path).resolve()
    path = str(path)
    sys.path.insert(0, path)
    try:
        yield
    finally:
        # Better to remove by value, in case `sys.path` was manipulated in between.
        sys.path.remove(path)


# Borrowed from AllenNLP
def import_module_and_submodules(
    package_name: str, exclude: Optional[Set[str]] = None
) -> None:
    """
    Import all submodules under the given package.
    Primarily useful so that people using AllenNLP as a library
    can specify their own custom packages and have their custom
    classes get loaded and registered.
    """
    if exclude and package_name in exclude:
        return

    importlib.invalidate_caches()

    # For some reason, python doesn't always add this by default to your path, but you pretty much
    # always want it when using `--include-package`.  And if it's already there, adding it again at
    # the end won't hurt anything.
    with push_python_path("."):
        # Import at top level
        module = importlib.import_module(package_name)
        path = getattr(module, "__path__", [])
        path_string = "" if not path else path[0]

        # walk_packages only finds immediate children, so need to recurse.
        for module_finder, name, _ in pkgutil.walk_packages(path):
            # Sometimes when you import third-party libraries that are on your path,
            # `pkgutil.walk_packages` returns those too, so we need to skip them.
            if path_string and module_finder.path != path_string:
                continue
            subpackage = f"{package_name}.{name}"
            import_module_and_submodules(subpackage, exclude=exclude)


def build_argument_parser():
    # Ensure all of the subcommands have been loaded. We do not want to load
    # the _models directory because it has code that goes into the Dockerfile, not the
    # repro project. Each model's own import (under repro/models/<name>) will import only
    # the files that are necessary from _models
    import_module_and_submodules("repro", exclude={"repro.models._models"})

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    # Add all of the root-level commands using the registry
    for name, (cls_, _) in sorted(Registrable._registry[RootSubcommand].items()):
        cls_().add_subparser(subparsers)

    return parser


def main():
    parser = build_argument_parser()
    args = parser.parse_args()
    if "func" in dir(args):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand


@SetupSubcommand.register("papineni2002")
class Papineni2002SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("papineni2002", f"{MODELS_ROOT}/papineni2002")



import logging
import os
from typing import Dict, List

from overrides import overrides

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import write_to_text_file
from repro.models import Model, SingleDocumentSummarizationModel
from repro.models.model import DocumentType, SummaryType

logger = logging.getLogger(__name__)


@Model.register("lewis2020-bart")
class BART(SingleDocumentSummarizationModel):
    """
    A wrapper around the BART model from `Lewis et al. (2020) <https://arxiv.org/abs/1910.13461>`.
    Currently, only the models trained on the CNN/DailyMail and XSum datasets are supported.
    """

    def __init__(
        self,
        model: str = "bart.large.cnn",
        batch_size: int = None,
        image: str = "lewis2020",
        device: int = 0,
    ) -> None:
        """
        Parameters
        ----------
        model : str, default="bart.large.cnn"
            The name of the pretrained model, either "bart.large.cnn" or "bart.large.xsum"
        batch_size : int, default=None
            The batch size for prediction. If `None`, defaults to the BART code's default.
        image : str, default="lewis2020"
            The name of the Docker image to run prediction in
        device : int, default=0
            The ID of the GPU to use, -1 if CPU
        """
        if model not in ["bart.large.cnn", "bart.large.xsum"]:
            raise Exception(f"Unknown pretrained model: {model}")
        self.model = model
        self.batch_size = batch_size
        self.image = image
        self.device = device

    @overrides
    def predict_batch(
        self, inputs: List[Dict[str, DocumentType]], *args, **kwargs
    ) -> List[SummaryType]:
        documents = [inp["document"] for inp in inputs]
        logger.info(
            f"Predicting summaries for {len(documents)} documents with Docker image {self.image}"
        )

        with TemporaryDirectory() as temp:
            input_dir = f"{temp}/input"
            output_dir = f"{temp}/output"
            volume_map = make_volume_map(input_dir, output_dir)

            host_input_file = f"{input_dir}/documents.txt"
            container_input_file = f"{volume_map[input_dir]}/documents.txt"
            write_to_text_file(documents, host_input_file)

            # Run inference. The output_dir must exist before running
            # the docker command
            os.makedirs(output_dir)
            host_output_file = f"{output_dir}/summaries.txt"
            container_output_file = f"{volume_map[output_dir]}/summaries.txt"
            command = (
                f"cd fairseq && "
                f"CUDA_VISIBLE_DEVICES={self.device} "
                f"python examples/bart/summarize.py"
                f"  --model-dir ../{self.model}"
                f"  --model-file model.pt"
                f"  --src {container_input_file}"
                f"  --out {container_output_file}"
            )

            if self.batch_size is not None:
                command += f" --bsz {self.batch_size}"

            if self.model == "bart.large.xsum":
                command += " --xsum-kwargs"

            cuda = self.device != -1
            run_command(self.image, command, volume_map=volume_map, cuda=cuda)

            # Load the output summaries
            summaries = open(host_output_file, "r").read().splitlines()
            return summaries



import json
import logging
import os
from typing import Dict, List

from overrides import overrides

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file
from repro.models import Model, RecipeGenerationModel

logger = logging.getLogger(__name__)


@Model.register("dugan2020-roft-recipe")
class RoFTRecipeGenerator(RecipeGenerationModel):
    def __init__(
        self,
        image: str = "dugan2020",
        device: int = 0,
        top_p: float = 0.7,
        repetition_penalty: float = 1.2,
        max_length: int = 256,
        random_seed: int = 4,
    ) -> None:
        self.model = "gpt2-xl"
        self.image = image
        self.device = device
        self.top_p = top_p
        self.repetition_penalty = repetition_penalty
        self.max_length = max_length
        self.random_seed = random_seed

    @overrides
    def predict_batch(self, inputs: List[Dict[str, str]], *args, **kwargs) -> List[str]:
        logger.info(
            f"Generating recipes for {len(inputs)} inputs and Docker image {self.image}"
        )

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"

            # Serialize the input to a file
            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for inp in inputs:
                    out.write(
                        json.dumps(
                            {"name": inp["name"], "ingredients": inp["ingredients"]}
                        )
                        + "\n"
                    )

            # Run inference. The output_dir must exist before running
            # the docker command
            os.makedirs(host_output_dir)
            host_output_file = f"{host_output_dir}/output.jsonl"
            container_output_file = f"{container_output_dir}/output.jsonl"

            command = (
                f"python generate_recipes.py"
                f"  --input-file {container_input_file}"
                f"  --model-name {self.model}"
                f"  --top-p {self.top_p}"
                f"  --repetition-penalty {self.repetition_penalty}"
                f"  --max-length {self.max_length}"
                f"  --random-seed {self.random_seed}"
                f"  --output-file {container_output_file}"
            )

            cuda = self.device != -1
            if cuda:
                command = f"CUDA_VISIBLE_DEVICES={self.device} " + command

            run_command(
                self.image,
                command,
                volume_map=volume_map,
                network_disabled=True,
                cuda=cuda,
            )

            results = read_jsonl_file(host_output_file, single_line=False)
            recipes = [result["recipe"] for result in results]
            return recipes



from overrides import overrides
from typing import List

from repro.common.io import read_jsonl_file
from repro.data.dataset_readers import DatasetReader
from repro.data.types import InstanceDict


@DatasetReader.register("chen2020-eval")
class Chen2020EvaluationDatasetReader(DatasetReader):
    @overrides
    def _read(self, *input_files: str) -> List[InstanceDict]:
        instances = []
        for input_file in input_files:
            predictions = read_jsonl_file(input_file)
            for prediction in predictions:
                instances.append(
                    {
                        "dataset": prediction["input"]["constituent_dataset"],
                        "source": prediction["input"]["metadata"]["source"],
                        "score": prediction["input"]["score"],
                        "prediction": prediction["prediction"],
                    }
                )
        return instances



# Pre-cache the BERT model so that it does not need to be downloaded
# every time the Dockerfile runs. We point to the cache_dir that the code
# uses by default.
from pytorch_transformers import BertModel, BertTokenizer

# The BertTokenizer is loaded with a specific cache and the default cache
BertModel.from_pretrained("bert-base-uncased", cache_dir="PreSumm/temp")
BertTokenizer.from_pretrained("bert-base-uncased", cache_dir="PreSumm/temp")
BertTokenizer.from_pretrained("bert-base-uncased")



import json
import os
from overrides import overrides
from typing import Any, List

from repro.data.output_writers import OutputWriter
from repro.data.types import InstanceDict


@OutputWriter.register("sacrerouge")
class SacreROUGEOutputWriter(OutputWriter):
    """
    Writes a jsonl file that is compatible for scoring with SacreROUGE. Each object
    includes an `instance_id`, `summarizer_id` (equal to the model name), `summarizer_type` (equal to "peer"),
    and `summary` (equal to the prediction). If the instance includes a reference, it will
    be included as well
    """

    def __init__(self):
        super().__init__(True)

    @overrides
    def _write(
        self,
        instances: List[InstanceDict],
        predictions: Any,
        output_file_or_dir: str,
        model_name: str,
        *args,
        **kwargs
    ) -> None:
        output_file = output_file_or_dir
        dirname = os.path.dirname(output_file)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        with open(output_file, "w") as out:
            for instance, prediction in zip(instances, predictions):
                output_dict = {
                    "instance_id": instance["instance_id"],
                    "summarizer_id": model_name,
                    "summarizer_type": "peer",
                    "summary": {"text": prediction},
                }
                if "reference" in instance:
                    output_dict["reference"] = {"text": instance["reference"]}
                elif "references" in instance:
                    output_dict["references"] = [
                        {"text": reference} for reference in instance["references"]
                    ]

                out.write(json.dumps(output_dict) + "\n")



import argparse
import json

from qaeval import QAEval


def main(args):
    kwargs = json.loads(args.kwargs)
    metric = QAEval(
        generation_model_path="models/generation/model.tar.gz",
        answering_model_dir="models/answering",
        lerc_model_path="models/lerc/model.tar.gz",
        lerc_pretrained_model_path="models/lerc/pretraining.tar.gz",
        **kwargs
    )

    candidates = []
    references_list = []
    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            candidates.append(data["candidate"])
            references_list.append(data["references"])

    results = metric.score_batch(candidates, references_list, return_qa_pairs=True)

    with open(args.output_file, "w") as out:
        for metrics, qa_pairs in results:
            out.write(json.dumps({"metrics": metrics, "qa_pairs": qa_pairs}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--kwargs", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import argparse
from overrides import overrides

from repro.common import Registrable
from repro.commands.subcommand import RootSubcommand, SetupSubcommand


@RootSubcommand.register("setup")
class SetupModelSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = "Setup a model"
        self.parser = parser.add_parser(
            "setup", description=description, help=description
        )
        subparsers = self.parser.add_subparsers()

        # Add all of the setup commands using the registry
        for name, (cls_, _) in sorted(Registrable._registry[SetupSubcommand].items()):
            cls_().add_subparser(subparsers)

        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        if "subfunc" in dir(args):
            args.subfunc(args)
        else:
            self.parser.print_help()



import json
import logging
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("scialom2021-questeval")
class QuestEval(Model):
    def __init__(self, image: str = "scialom2021", device: int = 0):
        self.image = image
        self.device = device

    @staticmethod
    def _check_single_text(texts_list: List[List[TextType]]) -> List[TextType]:
        single_texts = []
        for texts in texts_list:
            if texts is None:
                single_texts.append(None)
            else:
                if len(texts) != 1:
                    raise Exception(
                        f"QuestEval only supports single sources and references. Found: {len(texts)}"
                    )
                single_texts.append(texts[0])
        return single_texts

    def predict(
        self,
        candidate: TextType,
        sources: List[TextType] = None,
        references: List[TextType] = None,
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "sources": sources, "references": references}],
            **kwargs,
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating QuestEval for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        sources_list = [inp["sources"] if "sources" in inp else None for inp in inputs]
        references_list = [
            inp["references"] if "references" in inp else None for inp in inputs
        ]

        # QuestEval only supports single sources and references
        sources = self._check_single_text(sources_list)
        references = self._check_single_text(references_list)

        # Ensure all are strings or None
        candidates = [util.flatten(candidate) for candidate in candidates]
        sources = [
            util.flatten(source) if source is not None else None for source in sources
        ]
        references = [
            util.flatten(reference) if reference is not None else None
            for reference in references
        ]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, source, reference in zip(
                    candidates, sources, references
                ):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "source": source,
                                "reference": reference,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            kwargs_str = json.dumps(kwargs)
            if "'" in kwargs_str:
                raise Exception(
                    "Character `'` is currently not supported in values of `kwargs`"
                )

            cuda = self.device != -1
            commands = []
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
                score_device = 0
            else:
                score_device = -1

            commands.append(
                f"python score.py"
                f"  --input-file {container_input_file}"
                f"  --kwargs '{kwargs_str}'"
                f"  --cuda-device {score_device}"
                f"  --output-file {container_output_file}"
            )

            command = " && ".join(commands)
            backend.run_command(command=command, cuda=cuda, network_disabled=True)
            results = read_jsonl_file(host_output_file)
            micro_metrics = [{"questeval": metrics["scores"]} for metrics in results]
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics


@Model.register("scialom2021-questeval-summarization")
class QuestEvalForSummarization(QuestEval):
    @staticmethod
    def _check_and_update_kwargs(kwargs: Dict):
        if "task" in kwargs:
            if kwargs["task"] != "summarization":
                raise Exception(f'kwarg `task` must be equal to "summarization"')
        else:
            kwargs["task"] = "summarization"

        if "do_weighter" in kwargs:
            if kwargs["do_weighter"] != True:
                raise Exception(f"kwarg `do_weighter` must be equal to `True`")
        else:
            kwargs["do_weighter"] = True

    def predict(self, *args, **kwargs) -> MetricsType:
        self._check_and_update_kwargs(kwargs)
        return super().predict(*args, **kwargs)

    def predict_batch(self, *args, **kwargs) -> Tuple[MetricsType, List[MetricsType]]:
        self._check_and_update_kwargs(kwargs)
        return super().predict_batch(*args, **kwargs)


@Model.register("scialom2021-questeval-simplification")
class QuestEvalForSimplification(QuestEval):
    @staticmethod
    def _check_and_update_kwargs(kwargs: Dict):
        if "task" in kwargs:
            if kwargs["task"] != "text_simplification":
                raise Exception(f'kwarg `task` must be equal to "text_simplification"')
        else:
            kwargs["task"] = "text_simplification"

        if "do_BERTScore" in kwargs:
            if kwargs["do_BERTScore"] != True:
                raise Exception(f"kwarg `do_BERTScore` must be equal to `True`")
        else:
            kwargs["do_BERTScore"] = True

    def predict(self, *args, **kwargs) -> MetricsType:
        self._check_and_update_kwargs(kwargs)
        return super().predict(*args, **kwargs)

    def predict_batch(self, *args, **kwargs) -> Tuple[MetricsType, List[MetricsType]]:
        self._check_and_update_kwargs(kwargs)
        return super().predict_batch(*args, **kwargs)



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand


@SetupSubcommand.register("dugan2020")
class Dugan2020SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("dugan2020", f"{MODELS_ROOT}/dugan2020")



import json
import unittest
from parameterized import parameterized

from repro.models.dugan2020 import RoFTRecipeGenerator
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT


class TestDugan2020(unittest.TestCase):
    def setUp(self) -> None:
        self.expected_outputs = json.load(open(f"{FIXTURES_ROOT}/expected-output.json"))

    @parameterized.expand(get_testing_device_parameters())
    def test_roft_recipe_generator(self, device: int):
        model = RoFTRecipeGenerator(device=device)
        predictions = model.predict_batch(self.expected_outputs)
        device = "cpu" if device == -1 else "gpu"
        recipes = [inp["recipe"][device] for inp in self.expected_outputs]
        assert predictions == recipes
